/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.modules.api.controller;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.AllArgsConstructor;
import org.springblade.common.config.RedisConfig;
import org.springblade.common.constant.CommonConstant;
import org.springblade.common.enums.TicketTypeEnum;
import org.springblade.common.utils.RedisTool;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.modules.platform.service.IUserTicketService;
import org.springblade.modules.platform.service.UserAppService;
import org.springblade.modules.shopping.entity.IntegerConsign;
import org.springblade.modules.shopping.service.IIntegerConsignService;
import org.springblade.modules.shopping.vo.IntegerConsignVO;
import org.springblade.modules.system.entity.User;
import org.springblade.modules.system.entity.UserApp;
import org.springblade.modules.system.service.IUserService;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

/**
 * 积分寄售表 控制器
 *
 * @author BladeX
 * @since 2022-08-15
 */
@RestController
@AllArgsConstructor
@RequestMapping(CommonConstant.API_URL + "/integerConsign")
@Api(value = "积分寄售表", tags = "01积分寄售手机端接口")
public class IntegerConsignApiController extends BladeController {

	@Resource
	private IIntegerConsignService integerConsignService;

	@Resource
	private IUserService userService;

	@Resource
	private UserAppService userAppService;

	@Resource
	private RedisTool redisTool;

	@Resource
	private IUserTicketService userTicketService;

	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情", notes = "传入integerConsign")
	public R<IntegerConsign> detail(IntegerConsign integerConsign) {
		IntegerConsign detail = integerConsignService.getOne(Condition.getQueryWrapper(integerConsign));
		return R.data(detail);
	}

	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "寄售分页", notes = "传入integerConsign")
	public R<List<IntegerConsignVO>> page(IntegerConsignVO integerConsign, Query query) {
		Integer type = integerConsign.getType();
		if (type == 1){
			integerConsign.setConsignUserId(getUser().getUserId());
		}else if (type == 2) {
			integerConsign.setBuyerUserId(getUser().getUserId());
		}else{
			integerConsign.setStatus(0);
		}
		query.setDescs("create_time");
		IPage<IntegerConsignVO> pages = integerConsignService.selectIntegerConsignPage(Condition.getPage(query), integerConsign);
		return R.data(pages.getRecords());
	}

	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "修改", notes = "传入integerConsign")
	public R update(@Valid @RequestBody IntegerConsign integerConsign) {
		try {
			if(!redisTool.getRedisLock(integerConsign.getId().toString(), AuthUtil.getUserId().toString())){
				return R.fail("商品正在购买中，无法修改");
			}
			integerConsign.setIntegral(null);
			if(integerConsign.getConsignPrice().compareTo(integerConsign.getIntegral()) != 0){
				return R.fail("积分比例必须为1:1才能上架，需删除后重新上架");
			}
			integerConsignService.updateById(integerConsign);
			return R.success("修改成功");
		} catch (Exception e){
			e.printStackTrace();
			return R.fail("修改失败请稍后再试");
		}finally {//最后进行解锁
			redisTool.unRedisLock(integerConsign.getId().toString());
		}
	}

	@PostMapping("/release")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "发布积分寄售", notes = "传入integerConsign")
	@Transactional
	public R release(@Valid @RequestBody IntegerConsign integerConsign) {
		User user = userService.getById(getUser().getUserId());
		UserApp userApp = userAppService.getByUserId(AuthUtil.getUserId());
		integerConsign.setConsignTel(getUser().getAccount());//用户电话
		integerConsign.setConsignUserName(user.getRealName());//用户名称
		integerConsign.setConsignUserId(user.getId());//用户id
		integerConsign.setStatus(0);
		integerConsign.setConsignPrice(integerConsign.getIntegral());
		if(integerConsign.getIntegral().compareTo(new BigDecimal(userApp.getTicket())) == 1){
			return R.fail("积分不足");
		}
//		BigDecimal bigDecimal = integerConsign.getConsignPrice().divide(integerConsign.getIntegral(), BigDecimal.ROUND_CEILING).setScale(2, BigDecimal.ROUND_UP);
//		if(integerConsign.getConsignPrice().compareTo(integerConsign.getIntegral()) != 0){
//			return R.fail("积分比例必须为1:1");
//		}
		return R.status(integerConsignService.updateByStatus(integerConsign, 0));
	}

	public static void main(String[] args) {
		System.out.println(new BigDecimal(2000).compareTo(new BigDecimal(1900)));
	}

	@PostMapping("/offSelf")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "积分寄售下架", notes = "传入下架id")
	public R offSelf(@ApiParam(value = "积分寄售记录id", required = true) @RequestParam Long id){
		try {
			if(!redisTool.getRedisLock(id.toString(), AuthUtil.getUserId().toString())){
				return R.fail("商品正在购买中，无法下架");
			}
			IntegerConsign integerConsign = integerConsignService.getById(id);
			if(integerConsign.getStatus() == 1){
				return R.fail("商品已被购买，无法下架");
			}
			integerConsign.setId(id);
			integerConsign.setStatus(2);
			boolean b = integerConsignService.updateById(integerConsign);
			redisTool.unRedisLock(id.toString());
			if(b){
				return R.success("下架成功");
			}else{
				return R.fail("下架失败");
			}
		} catch (Exception e){
			e.printStackTrace();
			return R.fail("下架失败请稍后再试");
		}finally {//最后进行解锁
			redisTool.unRedisLock(id.toString());
		}
	}

	@PostMapping("/del")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "积分寄售删除并回退积分", notes = "传入积分寄售id")
	@Transactional
	public R del(@ApiParam(value = "积分寄售记录id", required = true) @RequestParam Long id){
		try {
			if(!redisTool.getRedisLock(id.toString(), AuthUtil.getUserId().toString())){
				return R.fail("商品正在购买中，无法删除");
			}
			IntegerConsign integerConsign = integerConsignService.getById(id);
			if(ObjectUtil.isNull(integerConsign)){
				return R.fail("商品已删除,请不要重复操作");
			}
			if(integerConsign.getStatus() == 1){
				return R.fail("商品已被购买，无法删除");
			}
			integerConsign.setId(id);
			integerConsign.setStatus(2);
			integerConsign.setIsDelete(1);
			boolean b = integerConsignService.updateByStatus(integerConsign, 2);
			if(b){
				return R.success("删除成功");
			}else{
				return R.fail("删除失败");
			}
		} catch (Exception e){
			e.printStackTrace();
			return R.fail("删除失败请稍后再试");
		}finally {//最后进行解锁
			redisTool.unRedisLock(id.toString());
		}
	}

	@PostMapping("/buyInteger")
	@ApiOperationSupport(order = 7)
	@ApiOperation(value = "购买付款", notes = "传入积分寄售记录id")
	public R buyInteger(@ApiParam(value = "积分寄售记录id", required = true) @RequestParam Long id,
						@ApiParam(value = "支付密码", required = true) @RequestParam String payPassword){
		try {
			if(!redisTool.getRedisLock(id.toString(), AuthUtil.getUserId().toString())){
				return R.fail("网络繁忙请稍后再试");
			}
			if(!userAppService.verifyPayPassword(payPassword, AuthUtil.getUserId())){
				return R.fail("支付密码不正确！");
			}
			IntegerConsign integerConsign = integerConsignService.getById(id);
			//判断用户余额是否充足
			UserApp userApp = userAppService.getByUserId(AuthUtil.getUserId());
			if(integerConsign.getConsignPrice().compareTo(userApp.getNowMoney()) == 1){
				return R.fail("余额不足，无法购买");
			}
			integerConsign.setBuyerUserId(AuthUtil.getUserId());
			integerConsign.setBuyTime(new Date());
			integerConsign.setBuyerUserTel(AuthUtil.getUserAccount());
			integerConsign.setBuyerUserName(AuthUtil.getUserName());
			integerConsign.setStatus(1);
			//购买人获得积分
			integerConsignService.updateByStatus(integerConsign, 1);
			return R.success("购买成功");
		} catch (Exception e){
			e.printStackTrace();
			return R.fail("购买失败请稍后再试");
		}finally {//最后进行解锁
			redisTool.unRedisLock(id.toString());
		}
	}
}
