package org.springblade.modules.api.controller;


import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.springblade.common.constant.CommonConstant;
import org.springblade.common.enums.PayFlagEnum;
import org.springblade.core.log.annotation.ApiLog;
import org.springblade.core.log.logger.BladeLogger;
import org.springblade.core.tool.api.R;
import org.springblade.modules.api.service.IZhzxAliService;
import org.springblade.modules.finance.entity.Order;
import org.springblade.modules.finance.entity.TransactionFlow;
import org.springblade.modules.finance.service.IOrderService;
import org.springblade.modules.finance.service.ITransactionFlowService;
import org.springblade.modules.shopping.service.IPaymentRecordService;
import org.springframework.util.Assert;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.*;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * Author pangqijun
 * Date 2022/11/24
 * Description 中宏咨询支付宝支付API
 */
@Slf4j
@RestController
@RequestMapping(CommonConstant.API_URL + "/aliPay/zhzx")
@Api(value = "中宏咨询支付宝支付API", tags = "96.中宏咨询支付宝支付API")
public class ZhzxAliPayController {

	/**
	 * 普通购买回调地址
	 */
	private static final String ORDER_URL = "/api/v1/aliPay/zhzx/callback";

	/**
	 * 购物车购买回调地址
	 */
	private static final String CART_ORDER_URL = "/api/v1/aliPay/zhzx/callback/cart";

	@Resource
	private BladeLogger logger;
	@Resource
	private IOrderService orderService;
	@Resource
	private ITransactionFlowService transactionFlowService;
	@Resource
	private IPaymentRecordService paymentRecordService;
	@Resource
	private IZhzxAliService zhzxAliService;


	private final ExecutorService executorService = Executors.newFixedThreadPool(20);


	@ApiLog("交易初始化")
	@PostMapping("/init")
	@ApiOperation(value = "交易初始化", notes = "交易初始化，拉起支付，返回支付参数给APP端")
	@ApiOperationSupport(order = 1)
	public R init(@ApiParam(value = "订单号", required = true) @RequestParam String outTradeNo) {
		Order order = orderService.getOne(Wrappers.<Order>lambdaQuery().eq(Order::getOutTradeNo, outTradeNo));
		Assert.notNull(order, "订单不存在");
		if (!PayFlagEnum.WAIT_PAY.getValue().equals(order.getPayFlag())) {
			return R.fail("当前订单状态不能支付");
		}
		String body = zhzxAliService.init(outTradeNo, order.getPayFee().toString(), ORDER_URL);
		// 支付记录
		paymentRecordService.init(order, 0);
		return R.data(body);
	}

	@ApiLog("购物车发起交易初始化")
	@PostMapping("/init/cart")
	@ApiOperation(value = "购物车发起交易初始化", notes = "购物车发起交易初始化")
	@ApiOperationSupport(order = 2)
	public R initiate(@ApiParam(value = "流水号", required = true) @RequestParam String transactionNo) {
		TransactionFlow flow = transactionFlowService.lambdaQuery().eq(TransactionFlow::getTransactionNo, transactionNo).one();
		if (flow == null) {
			return R.fail("数据不存在");
		}
		String body = zhzxAliService.init(transactionNo, flow.getAmount().toString(), CART_ORDER_URL);
		return R.data(body);
	}

	@ApiLog("支付宝购买回调")
	@RequestMapping(value = "/callback", method = {RequestMethod.GET, RequestMethod.POST})
	@ApiOperation(value = "异步通知相应-购买回调", notes = "异步通知相应,通知的间隔频率一般是：4m、10m、10m、1h、2h、6h、15h")
	@ApiOperationSupport(order = 3)
	public String orderNotify(HttpServletRequest request) {
		//获取支付宝请求信息
		Map<String,String> params = convertRequestParamsToMap(request);
		String paramsJson = JSON.toJSONString(params);
		try {
			log.info("支付宝回调参数[{}]", paramsJson);
			logger.info("支付宝购买回调", "订单号: " + params.get("out_trade_no") + ",回调参数:" + paramsJson);

			// 验证证书
			boolean flag = zhzxAliService.rsaCertCheck(params);
			if (!flag) {
				log.info("支付宝回调签名认证失败，signVerified=false, paramsJson:{}", paramsJson);
				return "failure";
			}
			log.info("支付宝回调签名认证成功");

			final String outTradeNo = params.get("out_trade_no");
			final Order order = orderService.getByOutTradeNo(outTradeNo);
			// 验证数据
			zhzxAliService.check(params, order);
			// 更新支付记录
			paymentRecordService.success(outTradeNo);
			// 校验成功就立即返回success，另起线程处理，此时已经跟支付宝关系不大
			executorService.execute(() -> {
				//获取请求参数中的商户订单号
				String tradeStatus = params.get("trade_status");
				// 支付成功
				if ("TRADE_SUCCESS".equals(tradeStatus)) {
					// 处理支付成功逻辑
					orderService.payCommonSuccess(order, CommonConstant.ALI_PAY);
				} else {
					log.error("没有处理支付宝回调业务，支付宝交易状态：{},params:{}", tradeStatus, paramsJson);
				}
			});
			return "success";
		} catch (Exception e) {
			log.error("支付宝回调异常,paramsJson:{},errorMsg:{}", paramsJson, e.getMessage());
			return "failure";
		}
	}

	@ApiLog("购物车购买支付宝回调")
	@RequestMapping(value = "/callback/cart", method = {RequestMethod.GET, RequestMethod.POST})
	@ApiOperation(value = "异步通知相应-购物车购买回调", notes = "异步通知相应,通知的间隔频率一般是：4m、10m、10m、1h、2h、6h、15h")
	@ApiOperationSupport(order = 100)
	public String cartOrderNotify(HttpServletRequest request) {
		//获取支付宝请求信息
		Map<String,String> params = convertRequestParamsToMap(request);
		String paramsJson = JSON.toJSONString(params);
		try {
			log.info("购物车支付宝回调参数[{}]", paramsJson);
			logger.info("购物车支付宝购买回调", "订单号: " + params.get("out_trade_no") + ",回调参数:" + paramsJson);

			// 验证证书
			boolean flag = zhzxAliService.rsaCertCheck(params);
			if (!flag) {
				log.info("支付宝回调签名认证失败，signVerified=false, paramsJson:{}", paramsJson);
				return "failure";
			}
			log.info("支付宝回调签名认证成功");

			final String outTradeNo = params.get("out_trade_no");
			final TransactionFlow flow = transactionFlowService.lambdaQuery().eq(TransactionFlow::getTransactionNo, outTradeNo).one();
			// 验证数据
			zhzxAliService.check(params, flow);
			// 更新交易流水状态
			flow.setStatus(2);
			flow.setPaymentTime(new Date());
			flow.setPaymentType(CommonConstant.ALI_PAY);
			transactionFlowService.updateById(flow);

			final List<Order> list = orderService.lambdaQuery().eq(Order::getTransactionNo, outTradeNo).list();

			// 校验成功就立即返回success，另起线程处理，此时已经跟支付宝关系不大
			executorService.execute(() -> {
				//获取请求参数中的商户订单号
				String tradeStatus = params.get("trade_status");
				// 支付成功
				if ("TRADE_SUCCESS".equals(tradeStatus)) {
					// 处理支付成功逻辑
					for (Order order : list) {
						log.info("当前处理订单号[{}], payFlag:[{}]", order.getOutTradeNo(), order.getPayFlag());
						orderService.payCommonSuccess(order, CommonConstant.ALI_PAY);
					}
				} else {
					log.error("没有处理支付宝回调业务，支付宝交易状态：{},params:{}", tradeStatus, paramsJson);
				}
			});
			return "success";
		} catch (Exception e) {
			log.error("购物车支付宝回调异常,paramsJson:{},errorMsg:{}", paramsJson, e.getMessage());
			return "failure";
		}
	}

	/**
	 * 将request中的参数转换成Map
	 * @param request
	 * @return
	 */
	private static Map<String, String> convertRequestParamsToMap(HttpServletRequest request) {
		Map<String, String> retMap = new HashMap<>();
		Map requestParams = request.getParameterMap();
		for (Object o : requestParams.keySet()) {
			String name = (String) o;
			String[] values = (String[]) requestParams.get(name);
			String valueStr = "";
			for (int i = 0; i < values.length; i++) {
				valueStr = (i == values.length - 1) ? valueStr + values[i] : valueStr + values[i] + ",";
			}
			//乱码解决，这段代码在出现乱码时使用。
			//valueStr = new String(valueStr.getBytes("ISO-8859-1"), "utf-8");
			retMap.put(name, valueStr);
		}
		return retMap;
	}

}
