package org.springblade.modules.finance.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.exception.ExceptionUtils;
import org.springblade.common.constant.CommonConstant;
import org.springblade.common.enums.BusinessTypeEnum;
import org.springblade.common.enums.GoodsSourceEnum;
import org.springblade.common.enums.PayFlagEnum;
import org.springblade.common.enums.YesOrNoEnum;
import org.springblade.common.utils.CommonUtil;
import org.springblade.core.log.exception.ServiceException;
import org.springblade.core.log.logger.BladeLogger;
import org.springblade.modules.api.request.OrderGoodsSaveRequest;
import org.springblade.modules.api.service.IZhzxAliService;
import org.springblade.modules.finance.entity.Order;
import org.springblade.modules.finance.entity.OrderException;
import org.springblade.modules.finance.entity.OrderGoods;
import org.springblade.modules.finance.pay.entity.AlipayProperties;
import org.springblade.modules.finance.pay.entity.WxPayV3Bean;
import org.springblade.modules.finance.service.IOrderGoodsService;
import org.springblade.modules.finance.service.IOrderService;
import org.springblade.modules.finance.vo.OrderExceptionVO;
import org.springblade.modules.finance.mapper.OrderExceptionMapper;
import org.springblade.modules.finance.service.IOrderExceptionService;
import org.springblade.modules.redenvelopemall.entity.GoodsInfo;
import org.springblade.modules.redenvelopemall.entity.GoodsSpec;
import org.springblade.modules.redenvelopemall.service.IGoodsInfoService;
import org.springblade.modules.redenvelopemall.service.IGoodsSpecService;
import org.springblade.modules.redenvelopemall.service.IWeiPinShangService;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import javax.annotation.Resource;
import java.util.Date;

/**
 * 异常订单表 服务实现类
 *
 * @author BladeX
 * @since 2022-11-15
 */
@Slf4j
@Service
public class OrderExceptionServiceImpl extends ServiceImpl<OrderExceptionMapper, OrderException> implements IOrderExceptionService {

	@Resource
	private IOrderService orderService;
	@Resource
	private IOrderGoodsService orderGoodsService;
	@Resource
	private IWeiPinShangService weiPinShangService;
	@Resource
	private IGoodsInfoService goodsInfoService;
	@Resource
	private IGoodsSpecService goodsSpecService;
	@Resource
	private WxPayV3Bean wxPayV3Bean;
	@Resource
	private BladeLogger bladeLogger;
	@Resource
	private IZhzxAliService zhzxAliService;

	@Override
	public IPage<OrderExceptionVO> selectOrderExceptionPage(IPage<OrderExceptionVO> page, OrderExceptionVO orderException) {
		return page.setRecords(baseMapper.selectOrderExceptionPage(page, orderException));
	}

	@Async
	@Override
	public void saveExceptionOrder(Order order, String errMessage, String payCode, Throwable throwable) {
		try {
			if (order != null && order.getId() != null) {
				if (PayFlagEnum.WAIT_PAY.getValue().equals(order.getPayFlag())) {
					// 异常订单单独更新订单支付类型，方便查询
					this.updateOrder(order.getId(), payCode);
				}
				OrderException orderException = new OrderException();
				orderException.setErrorMessage(ExceptionUtils.getStackTrace(throwable));
				orderException.setOrderId(order.getId());
				orderException.setIsProcess(0);
				int errorType = 0;
				if (StrUtil.isNotBlank(errMessage)) {
					if (errMessage.contains("创建订单错误10000,请联系管理员")) {
						errorType = 1;//余额不足
					} else if (errMessage.contains("此区域不发货")) {
						errorType = 2;
					}
				}
				orderException.setErrorType(errorType);
				this.save(orderException);
			}
		} catch (Exception e) {
			log.error("保存异常订单信息失败_" + e.getMessage(), e);
		}
	}

	/**
	 * 异常订单单独更新订单支付类型和状态
	 * @param orderId
	 * @param payCode
	 */
	public void updateOrder(Long orderId, String payCode) {
		String settlementAccount = "";
		if (CommonConstant.WX_PAY.equals(payCode)) {
			settlementAccount = wxPayV3Bean.getMchId();
		} else if(CommonConstant.ALI_PAY.equals(payCode)) {
			settlementAccount = zhzxAliService.getPayeeUserId();
		}
		Date currDate = new Date();
		// 异常订单单独更新订单支付类型，方便查询
		orderService.lambdaUpdate().set(Order::getPayFlag, PayFlagEnum.WAIT_DELIVERY.getValue())
			.set(Order::getSettlementAccount, settlementAccount)
			.set(Order::getPayTime, currDate)
			.set(Order::getUpdateTime, currDate)
			.set(Order::getPayCode, payCode).set(Order::getUpdateTime, currDate).eq(Order::getId, orderId).update();
	}

	@Override
	public void replacementOrder(String orderNo, String payCode) {
		Order order = orderService.lambdaQuery().eq(Order::getOutTradeNo, orderNo).one();
		Assert.notNull(order, "订单数据不存在");
		if (order.getPayFlag() > PayFlagEnum.WAIT_DELIVERY.getValue()) {
			throw new ServiceException("当前订单不支持补单");
		}
		orderService.payCommonSuccess(order, payCode);
	}

	@Transactional(rollbackFor = Exception.class)
	@Override
	public void reSubmitOrder(String oldOrderNo, String payCode) {
		Order order = orderService.lambdaQuery().eq(Order::getOutTradeNo, oldOrderNo).one();
		Assert.notNull(order, "订单数据不存在");
		//生成新的订单号
		String orderNo = CommonUtil.genTimeID();

		// 复制老订单数据
		Order newOrder = new Order();
		BeanUtil.copyProperties(order, newOrder);
		newOrder.setId(null);
		newOrder.setCreateTime(new Date());
		newOrder.setUpdateTime(new Date());
		newOrder.setOutTradeNo(orderNo);
		newOrder.setPayFlag(PayFlagEnum.WAIT_PAY.getValue());

		// 老的订单改为已取消
		order.setPayFlag(PayFlagEnum.CANCEL.getValue());
		orderService.updateById(order);

		// 复制老订单商品数据
		OrderGoods oldOrderGoods = orderGoodsService.lambdaQuery().eq(OrderGoods::getOutTradeNo, oldOrderNo).one();
		OrderGoods newOrderGoods = BeanUtil.copyProperties(oldOrderGoods, OrderGoods.class);
		newOrderGoods.setId(null);
		newOrderGoods.setOutTradeNo(orderNo);
		orderGoodsService.save(newOrderGoods);

		bladeLogger.info("重新下单", "原订单号:" + oldOrderNo + ",新订单号:" + orderNo);
		// 调用唯品尚下单接口
		GoodsInfo goodsInfo = goodsInfoService.getById(newOrderGoods.getGoodsId());
		Assert.notNull(goodsInfo, "goodsInfo is null");
		if (BusinessTypeEnum.NONE_SELF_SUPPORT.getValue().equals(goodsInfo.getBusinessType())
			&& GoodsSourceEnum.WPS.getValue().equals(goodsInfo.getSource())) {
			GoodsSpec goodsSpec = goodsSpecService.getById(newOrderGoods.getGoodsSpecIds());
			Assert.notNull(goodsSpec, "goodsSpec is null");
			String thirdOrderNo = weiPinShangService.createOrder(orderNo, goodsSpec.getOptionId(), goodsInfo.getThirdId(), newOrderGoods.getGoodsNum(), newOrder.getUserAddressId());
			newOrder.setThirdOrderNo(thirdOrderNo);
		}
		orderService.save(newOrder);

		// 调用支付成功接口
		orderService.payCommonSuccess(newOrder, payCode);
	}
}
