/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.modules.finance.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alipay.api.AlipayApiException;
import com.alipay.api.AlipayClient;
import com.alipay.api.request.AlipayTradeRefundRequest;
import com.alipay.api.response.AlipayTradeRefundResponse;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springblade.common.cache.ParamCache;
import org.springblade.common.constant.CommonConstant;
import org.springblade.common.constant.ConsignConstant;
import org.springblade.common.enums.*;
import org.springblade.common.utils.CommonUtil;
import org.springblade.common.utils.RedisTool;
import org.springblade.core.log.exception.ServiceException;
import org.springblade.core.log.logger.BladeLogger;
import org.springblade.core.redis.cache.BladeRedis;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.DateUtil;
import org.springblade.modules.api.request.OrderCommonSaveRequest;
import org.springblade.modules.api.request.OrderConsignRandomRequest;
import org.springblade.modules.api.request.OrderGoodsSaveRequest;
import org.springblade.modules.api.request.OrderTakeRequest;
import org.springblade.modules.api.response.ConsignResponse;
import org.springblade.modules.api.service.IAliService;
import org.springblade.modules.api.service.IWxPayService;
import org.springblade.modules.api.service.IZhzxAliService;
import org.springblade.modules.api.vo.LogisticVO;
import org.springblade.modules.courier.entity.CourierData;
import org.springblade.modules.courier.service.CourierNumberService;
import org.springblade.modules.finance.entity.*;
import org.springblade.modules.finance.mapper.OrderConsignMapper;
import org.springblade.modules.finance.mapper.OrderGrantMapper;
import org.springblade.modules.finance.mapper.OrderMapper;
import org.springblade.modules.finance.pay.entity.WxPayV3Bean;
import org.springblade.modules.finance.service.IOrderExceptionService;
import org.springblade.modules.finance.service.IOrderGoodsService;
import org.springblade.modules.finance.service.IOrderRefundService;
import org.springblade.modules.finance.service.IOrderService;
import org.springblade.modules.finance.vo.OrderVO;
import org.springblade.modules.finance.vo.PieVO;
import org.springblade.modules.linked.service.ILinkedOrderService;
import org.springblade.modules.platform.entity.UserScores;
import org.springblade.modules.platform.entity.UserTicket;
import org.springblade.modules.platform.mapper.UserScoresMapper;
import org.springblade.modules.platform.service.IUserTicketService;
import org.springblade.modules.redenvelopemall.entity.EarningsDetail;
import org.springblade.modules.redenvelopemall.entity.GoodsInfo;
import org.springblade.modules.redenvelopemall.entity.GoodsSpec;
import org.springblade.modules.redenvelopemall.entity.RedSetting;
import org.springblade.modules.redenvelopemall.mapper.GoodsSpecMapper;
import org.springblade.modules.redenvelopemall.service.*;
import org.springblade.modules.shopping.entity.*;
import org.springblade.modules.shopping.mapper.*;
import org.springblade.modules.shopping.wrapper.ConsignWrapper;
import org.springblade.modules.system.entity.User;
import org.springblade.modules.system.entity.UserApp;
import org.springblade.modules.system.mapper.UserAppMapper;
import org.springblade.modules.system.mapper.UserMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import javax.annotation.Resource;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.*;

/**
 * 订单表 服务实现类
 *
 * @author BladeX
 * @since 2022-02-13
 */
@Service
public class OrderServiceImpl extends ServiceImpl<OrderMapper, Order> implements IOrderService {

	private static final Logger log = LoggerFactory.getLogger(OrderServiceImpl.class);

	@Resource
	private UserMapper userMapper;

	@Resource
	private UserAppMapper userAppMapper;

	@Resource
	private GoodsMapper goodsMapper;

	@Resource
	private GoodsSpecsMapper goodsSpecsMapper;

	@Resource
	private SpecItemsMapper specItemsMapper;

	@Resource
	private ConsignMapper consignMapper;

	@Resource
	private OrderConsignMapper orderConsignMapper;

	@Resource
	private ConsignConfigMapper consignConfigMapper;

	@Resource
	private OrderGrantMapper orderGrantMapper;

	@Resource
	private UserScoresMapper userScoresMapper;

	@Autowired
	private IUserTicketService userTicketService;

	@Autowired
	private BladeLogger logger;

	@Resource
	private BladeRedis bladeRedis;
	@Resource
	private IOrderGoodsService orderGoodsService;
	@Resource
	private IGoodsInfoService goodsInfoService;
	@Resource
	private IGoodsSpecService goodsSpecService;
	@Resource
	private IEarningsDetailService earningsDetailService;
	@Resource
	private ISupplyChainService supplyChainService;
	@Resource
	private AlipayClient alipayClient;
	@Resource
	private IWeiPinShangService weiPinShangService;
	@Resource
	private IOrderRefundService orderRefundService;
	@Resource
	private IAliService aliService;
	@Resource
	private IWxPayService wxPayService;
	@Resource
	private CourierNumberService courierNumberService;
	@Autowired
	private IRedSettingService redSettingService;
	@Resource
	private GoodsSpecMapper goodsSpecMapper;
	@Resource
	private WxPayV3Bean wxPayV3Bean;
	@Resource
	private ILinkedOrderService linkedOrderService;
	@Value("${spring.profiles.active}")
	private String env;
	@Resource
	private IZhzxAliService zhzxAliService;
	@Resource
	private IOrderExceptionService orderExceptionService;

	@Override
	public IPage<Order> selectOrderPage(IPage<Order> page, Order order) {
		return page.setRecords(baseMapper.selectOrderPage(page, order));
	}

	@Override
	public IPage<OrderVO> goodsOrder(IPage<OrderVO> page, OrderVO order, String integralType) {
		List<Integer> orderTypeList;
		if ("1".equals(integralType)) {// 红色积分类型
			orderTypeList = CollUtil.newArrayList(OrderTypeEnum.RE_ORDER.getValue(), OrderTypeEnum.RE_EXCHANGE.getValue(), OrderTypeEnum.MAKE.getValue(), OrderTypeEnum.PARTNER.getValue());
		} else if ("2".equals(integralType)) {// 绿色积分类型
			orderTypeList = CollUtil.newArrayList(OrderTypeEnum.NORMAL.getValue());
		} else {
			orderTypeList = CollUtil.newArrayList(OrderTypeEnum.NORMAL.getValue(), OrderTypeEnum.RE_ORDER.getValue(),
				OrderTypeEnum.RE_EXCHANGE.getValue(), OrderTypeEnum.MAKE.getValue(), OrderTypeEnum.PARTNER.getValue(), OrderTypeEnum.LINKED.getValue());
		}
		return page.setRecords(baseMapper.goodsOrder(page, order, orderTypeList));
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public  String commonOrderInsert(OrderCommonSaveRequest request, String tenantId) {
		UserApp userApp = new UserApp();
		UserApp query = userApp.selectOne(Wrappers.<UserApp>lambdaQuery().eq(UserApp::getUserId,request.getUserId()));
		if (query == null){
			throw new ServiceException("请重新登录！");
		}
		if (query.getState() != 1){
			throw new ServiceException("登录账号已冻结！");
		}
		User user = userMapper.selectById(query.getUserId());
		//生成订单号
		String outTradeNo = CommonUtil.genTimeID();
		Order order = new Order();
		order.setTenantId(tenantId);
		if(user != null){
			order.setUserName(request.getReceivingName());
		}
		order.setOutTradeNo(outTradeNo);
		order.setUserId(query.getUserId());
		order.setOrderType(1);//普通订单
		order.setPayFlag(1); //待付款
		order.setPayFee(request.getPayFee());
		order.setUserAddress(request.getUserAddress());
		order.setTelPhone(request.getTelPhone());
		order.setDistributeMode(request.getDistributeMode());
		order.setDeliveryStaff(request.getDeliveryStaff());
		order.setReceivingName(request.getReceivingName());
		order.setWlPrice(request.getWlPrice());
		Double totalFee = 0.0;
		Double totalTicket = 0.0;
		List<OrderGoodsSaveRequest> goodsList = request.getGoodsList();
		if (goodsList == null || goodsList.size() == 0){
			return "";
		}
		for (OrderGoodsSaveRequest item: goodsList) {
			Goods goods = goodsMapper.selectById(item.getGoodsId());
			if (goods == null){
				throw new ServiceException("商品不存在！");
			}
			if (goods.getIsSale() == 1){
				throw new ServiceException("该商品未上架！");
			}
			if (goods.getGoodsStock() == 0){
				throw new ServiceException("该商品库存不足！");
			}
			OrderGoods orderGoods = new OrderGoods();
			orderGoods.setGoodsImg(goods.getGoodsImg());
			Double goodsPrice = goods.getGoodsPrice();
			Double ticketNum = goods.getTicketNum();
			GoodsSpecs goodsSpecs = goodsSpecsMapper.selectById(item.getGoodsSpecIds());
			if (goodsSpecs != null){
				//获取商品规格数据
				orderGoods.setGoodsPrice(goodsSpecs.getSpecPrice());
				goodsPrice = goodsSpecs.getSpecPrice();
//				ticketNum = goodsSpecs.getTicketNum();
				SpecItems specItems = specItemsMapper.selectById(goodsSpecs.getSpecIds());
				if (specItems != null){
					orderGoods.setGoodsSpecNames(specItems.getItemName());
					if(orderGoods.getGoodsImg() != null){
						orderGoods.setGoodsImg(specItems.getItemImg());
					}
				}
			}else{
				//获取商品规格数据
				orderGoods.setGoodsPrice(goods.getGoodsPrice());
//				ticketNum = goods.getTicketNum();
				orderGoods.setGoodsSpecNames(goods.getGoodsName());
				orderGoods.setGoodsImg(goods.getGoodsImg());
			}
			orderGoods.setOutTradeNo(outTradeNo);
			orderGoods.setGoodsImg(goods.getGoodsImg());
			orderGoods.setGoodsName(goods.getGoodsName());
			orderGoods.setGoodsNum(item.getGoodsNum());
			orderGoods.setGoodsSpecIds(item.getGoodsSpecIds());
			orderGoods.setTicketVal(goods.getTicketNum());
			Double specPrice = NumberUtil.toDouble(NumberUtil.mul(goodsPrice, item.getGoodsNum()));
			totalFee = NumberUtil.add(totalFee, specPrice);
			Double specTicket = NumberUtil.toDouble(NumberUtil.mul(ticketNum, item.getGoodsNum()));
			totalTicket = NumberUtil.add(totalTicket, specTicket);
			boolean flg = orderGoods.insert();
			if (!flg){
				throw new ServiceException("操作失败！");
			}
		};
		order.setTotalFee(totalFee);
		order.setTotalTicket(totalTicket);
		int i = baseMapper.insert(order);
		if (i != 1){
			throw new ServiceException("操作失败！");
		}
		return outTradeNo;

	}

	@Override
	public boolean consignSelfTake(OrderTakeRequest request) {
		Consign consign = consignMapper.selectById(request.getConsignId());
		if(consign != null
			&& Objects.equals(consign.getConsignUserId(), request.getUserId())
			&& consign.getOrderId() != null){
			Order order = baseMapper.selectById(consign.getOrderId());
			if (order != null){
				Order param = new Order();
				param.setId(order.getId());
				param.setPayFlag(3); //待收货
				param.setSendTime(DateUtil.now());
				int i = baseMapper.updateById(order);
				return i == 1;
			}
		}
		return false;
	}

	/**
	 * 从缓存获取用户单数据
	 * @param consignConfigId
	 * @param userId
	 * @return
	 */
	private Consign getConsignByCache(Long consignConfigId, Long userId) {
		System.out.println("从缓存获取用户单数据 userId:" + userId);
		Object data = bladeRedis.rPop(ConsignConstant.CONSIGN_CACHE + consignConfigId);
		if (data == null) {
			return null;
		}
		String s = JSON.toJSONString(data);
		JSONArray objects = JSON.parseArray(s);
		Object o = objects.get(0);
		Consign consign = JSON.parseObject(o.toString(), Consign.class);
		// 如果是自己的则放入缓存，重新获取
		if (consign != null && consign.getConsignUserId().equals(userId)) {
			if (bladeRedis.get("my:consign:list:" + userId + ":" + consign.getId()) != null) {
				bladeRedis.lPush(ConsignConstant.CONSIGN_CACHE + consignConfigId, JSON.toJSONString(consign));
				return null;
			}
			bladeRedis.setEx("my:consign:list:" + userId + ":" + consign.getId(), consign.getId(), 7200L);
			System.out.println("重新获取用户单数据");
			bladeRedis.lPush(ConsignConstant.CONSIGN_CACHE + consignConfigId, JSON.toJSONString(consign));
			return getConsignByCache(consignConfigId, userId);
		}
		return consign;

	}



	/**
	 * 1 生成订单
	 * 2 调用确认接口----统一收单接口
	 * 3 你收到支付宝的回调----去打款（商户给用户打款.）、
	 * 4 打款完成----去改变 （订单表---单个抢购订单状态--完成--寄售中）
	 * 4.1 去改变（ 预授单表）  ---1 单----stauts --2
	 *   ------------------------5 单--- num --5 -1 4
	 * 5 返回数给前端app  ---同时告诉app  用户是需要再次做预授
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public String consignOrderInsert(OrderConsignRandomRequest request, UserApp userApp, String tenantId) {
		// 获取用户单数据，没获取到用户单则获取平台单
		Consign consign = getConsignByCache(request.getConsignConfigId(), request.getUserId());
		if (consign == null){
			log.info("未获取到用户单数据，去获取平台单");
			consign = consignMapper.platformConsign(request.getConsignConfigId(), request.getConsignId());
			//数据量判断
			if (ObjectUtil.isNull(consign)) {
				throw new ServiceException("亲，好物已抢完欢迎再来！");
			}
		}

		//为交割的寄售商品没有选择提货
		if (consign.getIsDelivery() == 0){
			throw new ServiceException("寄售商品未交割！");
		}
		//扣除用户绿色积分 添加明细
		//寄售金额 * 3% =  绿色积分的
		Double score = NumberUtil.mul(consign.getConsignPrice(), Double.valueOf(0.03));
		Double ratioTicket = NumberUtil.roundDown(score , 2).doubleValue();
		//用户当前积分数必须 大于 抢购的3%的积分数
		if(userApp.getTicket() < ratioTicket){
			throw new ServiceException("亲，您的绿色积分达不到进货条件！");
		}
		//查询商品信息
		Goods goods = goodsMapper.selectById(consign.getGoodsId());
		if (ObjectUtil.isNull(goods)){
			throw new ServiceException("商品不存在！");
		}
		User user = userMapper.selectById(request.getUserId());
		String outTradeNo = CommonUtil.genTimeID();
		boolean flg = createOrder(userApp, user,request,consign,tenantId,goods,score,ratioTicket,outTradeNo);
		if (flg){
			return outTradeNo;
		}
		return "";
	}

	/**
	 * 1、创建订单
	 * 2、创建订单详情
	 * 3、创建寄售订单
	 * 4、修改用户积分
	 * 5、添加积分明细
	 * @return boolean
	 */
	private boolean createOrder(UserApp userApp,
								User user,
								OrderConsignRandomRequest request,
								Consign consign,
								String tenantId,
								Goods goods,
								Double score,
								Double ratioTicket,
								String outTradeNo){
		//创建订单号
		Order order = new Order();
		order.setTenantId(tenantId);
		if(user != null){
			order.setUserName(user.getName());
		}
		order.setOutTradeNo(outTradeNo);
		order.setUserId(request.getUserId());
		order.setOrderType(3);//寄售订单
		order.setPayFlag(4); //寄售订单 变为已完成
		order.setPayFee(consign.getConsignPrice());
		order.setTotalFee(consign.getConsignPrice());
		order.setUserAddress(request.getUserAddress());
		order.setTelPhone(request.getTelPhone());
		order.setDistributeMode(2);//默认物流配送
		order.setDeliveryStaff("物流配送");
		order.setReceivingName(request.getReceivingName());
		order.setOrderState(1); //支付订单状态 发起中
		int i = baseMapper.insert(order);
		if (i == 1){
			//订单详情数据
			OrderGoods orderGoods = new OrderGoods();
			orderGoods.setOutTradeNo(outTradeNo);
			orderGoods.setGoodsImg(goods.getGoodsImg());
			orderGoods.setGoodsName(goods.getGoodsName());
			orderGoods.setGoodsNum(1);
			orderGoods.setGoodsPrice(consign.getConsignPrice());
			orderGoods.setTicketVal(consign.getConsignPrice());
			boolean flg = orderGoods.insert();
			if (!flg){
				throw new ServiceException("操作失败！");
			}
			//创建寄售订单数据
			OrderConsign orderConsign = new OrderConsign();
			orderConsign.setTotalNum(1);//默认寄售数量为1
			/**
			 * 抢到用户单
			 * 1、更新用户单状态
			 * 2、创建新的用户单
			 */
			Consign updateConsign = new Consign();
			if(consign.getConsignFor() == 2){
				updateConsign.setId(consign.getId());
				updateConsign.setConsignState(2); //待付款
				updateConsign.setConsignType(request.getConsignType());//修改寄售类型（寄售方式）
				updateConsign.setNowStock(0); //当前库存为0
			}else if(consign.getConsignFor() == 1){
				/**
				 * 抢到平台单
				 * 	1、变为待确认状态
				 * 	2、剩余数量为大于1时 寄售状态为还是寄售中 等于1是为已完成
				 * 	3、传新的用户单
				 */
				updateConsign.setId(consign.getId());
				updateConsign.setConsignType(request.getConsignType());
				if (consign.getNowStock() == 1){
					updateConsign.setNowStock(0);
				}else{
					updateConsign.setNowStock(consign.getNowStock() - 1);
				}
				orderConsign.setIsConfirm(5);
			}
			int c =  consignMapper.updateById(updateConsign);
			if (c != 1){
				throw new ServiceException("亲！请重新抢单");
			}
			orderConsign.setConsignId(consign.getId()); //寄售ID
			orderConsign.setOutTradeNo(outTradeNo);
			orderConsign.setGoodsId(consign.getGoodsId());
			orderConsign.setGoodsName(goods.getGoodsName());
			orderConsign.setConsignType(request.getConsignType());
			orderConsign.setPayUserId(request.getUserId()); //支付人
			if(request.getConsignType() == 1){
				orderConsign.setConsignDays(0);
			}else if(request.getConsignType() == 2){
				orderConsign.setConsignDays(consign.getConsignDays());
			}else{
				throw new ServiceException("请求参数错误！");
			}
			orderConsign.setConsignUserId(consign.getConsignUserId());
			orderConsign.setTotalPrice(consign.getConsignPrice());
			orderConsign.setConsignPrice(consign.getConsignPrice());
			int j = orderConsignMapper.insert(orderConsign);
			if(j != 1){
				throw new ServiceException("抢单失败！");
			}
			//更新用户绿色积分并添加明细
			if(user != null){
				Double ticket = NumberUtil.sub(userApp.getTicket() - score).doubleValue();
				//用户减去绿色积分（酒票）
				UserApp param = new UserApp();
				param.setId(userApp.getId());
				param.setTicket(ticket);
				flg = param.updateById();
				if (flg){
					//添加积分明细（酒票） 寄售扣除3%的手续费
					UserTicket userTicket = new UserTicket();
					userTicket.setDataSrc(2);//酒票转让
					userTicket.setServesRatio(0.03);
					userTicket.setTicket(ratioTicket);
					userTicket.setUserId(userApp.getUserId());
					userTicket.setUserTicket(userApp.getTicket());
					userTicket.setTicketType(2);//支出
					userTicket.setCreateTime(DateUtil.now());
					userTicket.setDataId(consign.getId()); //寄售ID
					userTicket.setDataRemarks("进货扣除");
					userTicket.setLastTicket(ticket); //最后酒票数
					userTicketService.save(userTicket);
				}
			}
			//每完成一次进货（+20信誉分）
			UserScores userScores = new UserScores();
			userScores.setUserId(userApp.getUserId());//进货
			int consignScore = Integer.parseInt(ParamCache.getValue(CommonConstant.SCORE_CONSIGN_EVERY_TIME));
			String dataRemarks = "完成进货";
			if (userApp.getCreditScore() >= 500){
				consignScore = Integer.parseInt(ParamCache.getValue(CommonConstant.SCORE_CONSIGN_OVER));
				userScores.setScore(consignScore);
				dataRemarks = "完成进货（500信誉分以上或含有500信誉分）";
			}
			userScores.setScore(consignScore);
			userScores.setDataSrc("2");
			userScores.setDataId(consign.getId()); //进货仓单
			userScores.setDataType(1);
			userScores.setDataRemarks(dataRemarks);
			userScoresMapper.insert(userScores);
			UserApp userAppUpdate = new UserApp();
			userAppUpdate.setId(userApp.getId());
			userAppUpdate.setCreditScore(userApp.getCreditScore() + consignScore);
			userAppUpdate.updateById();
		}
		return true;
	}

	@Override
	public Order getByOutTradeNo(String outTradeNo) {
		return this.lambdaQuery().eq(Order::getOutTradeNo, outTradeNo).one();
	}

	@Override
	public Map<String, Object> dataReport() {
		Map<String, Object> result = new HashMap<>();
		result.put("totalOrderCount", this.baseMapper.selectTotalOrderCount());
		result.put("totalOrderAmount", this.baseMapper.selectTotalOrderAmount());
		result.put("totalOrderCountToday", this.baseMapper.selectTotalOrderCountToday());
		result.put("totalOrderAmountToday", this.baseMapper.selectTotalOrderAmountToday());
		return result;
	}

	@Override
	public Map<String, Object> orderGoodsProportion(String queryType) {
		Map<String, Object> result = new HashMap<>();
		if ("full-year".equals(queryType)) {
			convertProportion(this.baseMapper.selectStaticByProportion(null), this.baseMapper.selectStaticByZone(null), result);
		} else if ("nearly_one".equals(queryType)) {
			convertProportion(this.baseMapper.selectStaticByProportion(60), this.baseMapper.selectStaticByZone(60), result);
		} else if ("nearly_tow".equals(queryType)) {
			convertProportion(this.baseMapper.selectStaticByProportion(30), this.baseMapper.selectStaticByZone(30), result);
		} else if ("week".equals(queryType)) {
			convertProportion(this.baseMapper.selectStaticByProportion(7), this.baseMapper.selectStaticByZone(7), result);
		}
		return result;
	}

	@Override
	public Map<String, Object> dataReportByDate(String queryType) {
		Map<String, Object> result = new HashMap<>();
		if ("full-year".equals(queryType)) {
			convertStitc(this.baseMapper.selectCurrYear(), result);
		} else if ("nearly_one".equals(queryType)) {
			convertStitc(this.baseMapper.selectStaticByDay(60), result);
		} else if ("nearly_tow".equals(queryType)) {
			convertStitc(this.baseMapper.selectStaticByDay(30), result);
		} else if ("week".equals(queryType)) {
			convertStitc(this.baseMapper.selectStaticByDay(7), result);
		}
		return result;
	}

	private void convertProportion(List<PieVO> source, List<PieVO> zone, Map<String, Object> result) {
		result.put("source", source);
		result.put("zone", zone);
	}

	private void convertStitc(List<Map<String, Object>> maps, Map<String, Object> result) {
		List<Object> desc = new ArrayList<>();
		List<Object> totalCount = new ArrayList<>();
		List<Object> totalAmount = new ArrayList<>();
		for (Map<String, Object> map : maps) {
			desc.add(map.get("dateValue"));
			totalCount.add(map.get("totalCount"));
			totalAmount.add(map.get("totalAmount"));
		}
		result.put("desc", desc);
		result.put("totalCount", totalCount);
		result.put("totalAmount", totalAmount);
	}

	@Override
	public void checkTimeout(Order order) {
		Date createTime = order.getCreateTime();
		if (createTime == null) {
			return;
		}
		Date date = new Date();
		long between = cn.hutool.core.date.DateUtil.between(createTime, date, DateUnit.SECOND);
		// 待支付的订单时间超过半小时，则关闭订单
		if (PayFlagEnum.WAIT_PAY.getValue().equals(order.getPayFlag()) && between > 1800) {
			if (order.getId() != null) {
				this.lambdaUpdate().set(Order::getPayFlag, PayFlagEnum.CANCEL.getValue())
					.set(Order::getUpdateTime, date).eq(Order::getId, order.getId()).update();
			}
			throw new ServiceException("订单已超时");
		}
	}

	@Override
	public void commonRefund(Order order) {
		boolean flag = false;
		// 支付宝退款
		if (CommonConstant.ALI_PAY.equals(order.getPayCode())) {
			if (zhzxAliService.getPayeeUserId().equals(order.getSettlementAccount())) {
				zhzxAliService.refund(order);
			} else {
				aliService.refund(order);
			}
			flag = true;
		} else if (CommonConstant.WX_PAY.equals(order.getPayCode())) { // 微信退款
			flag = wxPayService.orderRefund(order);
		} else {
			throw new ServiceException("暂不支持该退款方式" + order.getPayCode());
		}
		if (SettleTypeEnum.SELF_SUPPORT.getValue().equals(order.getSettleType()) && flag) {
			// 及时到账的订单需要扣除购买时获得的红色积分
			redIntegralService.refundDeduct(order);
		}
	}

	@Override
	public void coRichOutReward(Long orderId) {
		Order order = this.getById(orderId);
		Assert.notNull(order, "订单数据不存在");
		if ("dev".equals(env) || "test".equals(env)) {
			order.setPayFee(0.01);
		}
		if (CommonConstant.ALI_PAY.equals(order.getPayCode())) {
			aliService.refund(order);
		} else if (CommonConstant.WX_PAY.equals(order.getPayCode())) {
			wxPayService.orderRefund(order);
		} else {
			throw new ServiceException("占不支持该退款方式" + order.getPayCode());
		}
	}

	@Transactional(rollbackFor = Exception.class)
	@Override
	public void cityPartnerRefund(String orderNo, String userName) {
		Order order = this.lambdaQuery().eq(Order::getOutTradeNo, orderNo).one();
		if (order == null) {
			throw new ServiceException("订单数据不存在");
		}
		if (PayFlagEnum.REFUNDED.getValue().equals(order.getPayFlag())) {
			throw new ServiceException("已退款，请勿重复操作");
		}
		if (!OrderTypeEnum.PARTNER.getValue().equals(order.getOrderType())) {
			throw new ServiceException("城市合伙人订单暂不支持退款");
		}
		UserApp userApp = userAppMapper.selectOne(Wrappers.lambdaQuery(UserApp.class).eq(UserApp::getUserId, order.getUserId()));
		// 退回绿色积分
		Double ticket = NumberUtil.add(userApp.getTicket(), order.getNeedIntegral());
		userApp.setTicket(ticket);
		userApp.updateById();
		UserTicket userTicket = new UserTicket();
		userTicket.setDataSrc(1);//1.订单
		userTicket.setServesRatio(0D);
		userTicket.setTicket(order.getNeedIntegral());
		userTicket.setUserId(userApp.getUserId());
		userTicket.setUserTicket(userApp.getTicket());
		userTicket.setTicketType(TicketTypeEnum.INCOME.getValue());//收入
		userTicket.setCreateTime(DateUtil.now());
		userTicket.setDataId(order.getId()); //订单号
		userTicket.setDataRemarks("订单退回");
		userTicket.setLastTicket(ticket); //最后酒票数
		userTicketService.save(userTicket);
		// 修改角色
		userApp.setIsRole(userApp.getIsRole() - 1);
		userAppMapper.updateById(userApp);
		// 扣除红色积分
		redIntegralService.refundDeduct(order);
		order.setPayFlag(PayFlagEnum.REFUNDED.getValue());
		this.updateById(order);
	}

	@Override
	public List<LogisticVO> logistic(String orderNo) throws Exception {
		OrderGoods orderGoods = orderGoodsService.lambdaQuery().eq(OrderGoods::getOutTradeNo, orderNo).one();
		List<LogisticVO> logisticVOS = new ArrayList<>();
		Order order = this.lambdaQuery().eq(Order::getOutTradeNo, orderNo).one();
		if (order.getPayFlag().equals(PayFlagEnum.CANCEL.getValue()) || order.getPayFlag().equals(PayFlagEnum.WAIT_PAY.getValue())) {
			return logisticVOS;
		}
		if (Objects.nonNull(orderGoods)) {
			if (BusinessTypeEnum.NONE_SELF_SUPPORT.getValue().equals(orderGoods.getBusinessType())) {
				GoodsInfo goodsInfo = goodsInfoService.get(orderGoods.getGoodsId());
				Assert.notNull(goodsInfo, "商品数据不存在");
				GoodsSpec goodsSpec = goodsSpecMapper.get(orderGoods.getGoodsSpecIds());
				Assert.notNull(goodsSpec, "商品规格数据不存在");
				if (GoodsSourceEnum.STBZ.getValue().equals(goodsInfo.getSource())) {
					JSONObject logistic = supplyChainService.logistic(orderNo, goodsSpec.getOptionId());
					JSONObject data = logistic.getJSONObject("data");
					if (data != null && data.getJSONArray("list") != null) {
						// 将10位的时间戳转换为13位的
						for (Object o : logistic.getJSONObject("data").getJSONArray("list")) {
							JSONObject json = (JSONObject) o;
							json.put("time", json.getLongValue("time") * 1000);
						}
						logisticVOS = logistic.getJSONObject("data").getJSONArray("list").toJavaList(LogisticVO.class);
						logisticVOS.sort((a, b) -> b.getTime().compareTo(a.getTime()));
					}
				} else if (GoodsSourceEnum.WPS.getValue().equals(goodsInfo.getSource())) {
					JSONObject expressInfo = weiPinShangService.getExpressInfo(order.getThirdOrderNo());
					JSONObject data = expressInfo.getJSONObject("data");
					if (data != null) {
						JSONArray jsonArray = data.getJSONArray("Traces");
						for (Object o : jsonArray) {
							JSONObject jsonObject = (JSONObject) o;
							LogisticVO logisticVO = new LogisticVO();
							logisticVO.setMessage(jsonObject.getString("AcceptStation"));
							logisticVO.setTime(jsonObject.getDate("AcceptTime"));
							logisticVOS.add(logisticVO);
						}
						logisticVOS.sort((a, b) -> b.getTime().compareTo(a.getTime()));
					}
				} else {
					throw new ServiceException("商品来源属性错误");
				}
			} else {
				List<CourierData> courierDataList = courierNumberService.selectCourierData(orderGoods.getOutTradeNo());
				if (CollUtil.isNotEmpty(courierDataList)) {
					for (CourierData courierData : courierDataList) {
						LogisticVO logisticVO = new LogisticVO();
						logisticVO.setMessage(courierData.getContext());
						logisticVO.setTime(courierData.getFtime());
						logisticVOS.add(logisticVO);
					}
				}
			}
		}
//		OrderConsign orderConsign = orderConsignService.lambdaQuery().eq(OrderConsign::getOutTradeNo, orderNo).one();
//		if (Objects.nonNull(orderConsign)){
//
//			List<CourierData> courierDataList = courierNumberService.selectCourierData(orderGoods.getOutTradeNo());
//			if (courierDataList.size()<=0){
//				return R.data(new ArrayList<LogisticVO>());
//			}
//			List<LogisticVO> logisticVOS = new ArrayList<>();
//			for (CourierData courierData : courierDataList) {
//				LogisticVO logisticVO = new LogisticVO();
//				logisticVO.setMessage(courierData.getContext());
//				logisticVO.setTime(courierData.getFtime());
//				logisticVOS.add(logisticVO);
//			}
//			return R.data(logisticVOS);
//		}
		return logisticVOS;
	}

	@Override
	public ConsignResponse consignGrant(UserApp query, Long consignConfigId, String orderNo) {
		ConsignResponse response = new ConsignResponse();
		//已抢次数 （可抢订单数/日）
		Integer count = orderConsignMapper.selectUserCount(query.getUserId(), consignConfigId);
		log.info("用户【" + query.getUserId() + "】在场次【" + consignConfigId + "】已抢购次数为：" + count);
		OrderGrant orderGrant = orderGrantMapper.selectOne(Wrappers.<OrderGrant>lambdaQuery().eq(OrderGrant::getOrderNo, orderNo));
		log.info("用户【" + query.getUserId() + "】授权数据:" + orderGrant);
		//是特殊账号的不判断的抢购次数
		log.info("用户【" + query.getUserId() + "】为普通用户，判断信誉积分数据");
		double k = (double)query.getCreditScore() / 100; //可抢次数
		int scoreCount = NumberUtil.roundDown(k, 2).intValue();
		int consignCount = Integer.parseInt(ParamCache.getValue(CommonConstant.DEFAULT_CONDIGN_COUNT));
		//最多只能抢次数
		if (scoreCount > consignCount){
			scoreCount = consignCount;
		}
		log.info("用户【" + query.getUserId() + "】为信誉分可抢次数为：" + scoreCount);
		int num = scoreCount - count;
		if (query.getEnableAuth() != 2){
			if (num <= 0){
				response.setStatus(3);//已抢购完成
				response.setNum(0);
				response.setEnableCount(num);
				return response;
			}
		}
		if (orderGrant != null){
			//已解冻 则该账单已支付完成
			if (orderGrant.getStatus() == 2){
				response.setStatus(1);//未授权
				response.setNum(0); //剩余已授权的次数
				response.setEnableCount(num);
				return response;
			}
			List<Order> list = baseMapper.selectList(Wrappers.<Order>lambdaQuery().in(Order::getGrantNo, orderNo));
			log.info("用户【" + query.getUserId() + "】授权订单号【" + orderNo +"】已授权的关联的订单数为：" + list.size());
			log.info("用户【" + query.getUserId() + "】授权订单号【" + orderNo +"】的数量：" + orderGrant.getNum());
			//如果剩余次数为大于等于1
			// 订单表
			if(list.size() < orderGrant.getNum()){
				response.setStatus(2);//已授权
				response.setNum(orderGrant.getNowNum()); //剩余已授权的次数
				response.setEnableCount(num);
				return response;
			}
		}
		response.setStatus(1);//未授权
		response.setNum(0); //剩余已授权的次数
		response.setEnableCount(num);
		return response;
	}

	@Override
	public void refund(String orderNo, BigDecimal amount, BigDecimal deductIntegral) {
		Order order = this.lambdaQuery().eq(Order::getOutTradeNo, orderNo).one();
		if (order == null) {
			throw new ServiceException("订单数据不存在");
		}
		// 调用支付宝退款接口
		AlipayTradeRefundRequest request = new AlipayTradeRefundRequest();
		JSONObject bizContent = new JSONObject();
		bizContent.put("refund_amount", amount);
		bizContent.put("out_trade_no", order.getOutTradeNo());
		bizContent.put("out_request_no", "TK" + order.getOutTradeNo());
		request.setBizContent(bizContent.toString());
		try {
			AlipayTradeRefundResponse response = alipayClient.certificateExecute(request);
			if(response.isSuccess()){
				log.info("退款成功, 订单号[{}]", order.getOutTradeNo() + ",支付宝返回数据:" + JSON.toJSONString(response));
				bladeLogger.info("退款成功", "订单号:" + order.getOutTradeNo() + ",支付宝返回数据:" + JSON.toJSONString(response));
				order.setPayFlag(PayFlagEnum.REFUNDED.getValue());
				order.setIsRefund(YesOrNoEnum.YES.getValue());
				order.setRefundAmount(amount);
				order.setDeductIntegral(deductIntegral);
				this.updateById(order);
				order.setTotalTicket(deductIntegral.doubleValue());
				redIntegralService.refundDeduct(order);
			} else {
				log.info("退款失败, 订单号[{}], 支付宝返回数据[{}]", order.getOutTradeNo(), JSON.toJSONString(response));
				bladeLogger.info("退款失败", "订单号:" + order.getOutTradeNo() + ",支付宝返回数据:" + JSON.toJSONString(response));
				throw new ServiceException("退款失败, " + response.getCode() + ":" + response.getSubMsg());
			}
		} catch (AlipayApiException e) {
			log.error("退款异常," + e.getMessage(), e);
			bladeLogger.error("退款异常", "订单号:" + order.getOutTradeNo() + ",errorMessage:" + e.getMessage());
			throw new ServiceException(e.getMessage());
		}
	}

	@Transactional(rollbackFor = Exception.class)
	@Override
	public void refundCart(String orderNo, BigDecimal amount, BigDecimal deductIntegral) {
		Order order = this.lambdaQuery().eq(Order::getOutTradeNo, orderNo).one();
		if (order == null) {
			throw new ServiceException("订单数据不存在");
		}
		if (PayFlagEnum.COMPLETED.getValue().equals(order.getPayFlag())) {
			throw new ServiceException("已完成订单不支持退款");
		}
		if (PayFlagEnum.REFUNDED.getValue().equals(order.getPayFlag())) {
			throw new ServiceException("已退款，请勿重复操作");
		}
		if (PayFlagEnum.CANCEL.getValue().equals(order.getPayFlag())) {
			throw new ServiceException("该订单已取消，不能退款");
		}
		if (StrUtil.isBlank(order.getPayCode())) {
			throw new ServiceException("payCode参数异常");
		}
		if (OrderTypeEnum.LINKED.getValue().equals(order.getOrderType()) && Boolean.TRUE.equals(order.getLinkedResult())) {
			throw new ServiceException("当前订单不支持退款");
		}
		this.commonRefund(order);
	}

	@Transactional(rollbackFor = Exception.class)
	@Override
	public void orderRefund(Long id) {
		Order order = this.getById(id);
		if (order == null) {
			throw new ServiceException("订单数据不存在");
		}
		if (OrderTypeEnum.PARTNER.getValue().equals(order.getOrderType())) {
			throw new ServiceException("城市合伙人订单暂不支持退款");
		}
		if (!PayFlagEnum.WAIT_DELIVERY.getValue().equals(order.getPayFlag())) {
			throw new ServiceException("当前状态不支持退款");
		}
		if (OrderTypeEnum.LINKED.getValue().equals(order.getOrderType()) && Boolean.TRUE.equals(order.getLinkedResult())) {
			throw new ServiceException("当前订单不支持退款");
		}
		logger.info("发起退款", "订单号：" + order.getOutTradeNo() + "订单状态");
		OrderRefund orderRefund = OrderRefund.build(order.getId());
		List<OrderGoods> list = orderGoodsService.lambdaQuery().eq(OrderGoods::getOutTradeNo, order.getOutTradeNo()).list();
		if (CollUtil.isNotEmpty(list)) {
			OrderGoods orderGoods = list.get(0);
			// 非自营商品调用第三方退款
			if (BusinessTypeEnum.NONE_SELF_SUPPORT.getValue().equals(orderGoods.getBusinessType())) {
				GoodsSpec goodsSpec = goodsSpecMapper.get(orderGoods.getGoodsSpecIds());;
				GoodsInfo goodsInfo = goodsInfoService.getById(goodsSpec.getGoodsId());
				if (GoodsSourceEnum.STBZ.getValue().equals(goodsInfo.getSource())) {
					JSONObject jsonObject = supplyChainService.afterSaleBeforeCheck(orderGoods.getOutTradeNo(), goodsSpec.getOptionId());
					log.info("售后前置校验结果:[{}]", jsonObject.toJSONString());
					JSONArray jsonArray = jsonObject.getJSONObject("data").getJSONArray("serviceType");
					boolean flag = false;
					for (Object o : jsonArray) {
						JSONObject e = (JSONObject) o;
						if ("40".equals(e.getString("code"))) {
							flag = true;
							break;
						}
					}
					if (!flag) {
						orderRefundService.failure(orderRefund, "当前订单不支持退款");
						throw new ServiceException("当前订单不支持退款");
					}
					supplyChainService.afterSale(orderGoods.getOutTradeNo(), goodsSpec.getOptionId(), orderGoods.getGoodsNum(), order.getWlPrice(), "40");
				} else if (GoodsSourceEnum.WPS.getValue().equals(goodsInfo.getSource())) {
					try {
						weiPinShangService.createAfsApply(order.getThirdOrderNo(), "40", false, order.getUserAddressId());
					} catch (Exception e) {
						orderRefundService.failure(orderRefund, e.getMessage());
						throw new ServiceException(e.getMessage());
					}
				} else {
					throw new ServiceException("商品来源参数错误");
				}
				order.setPayFlag(PayFlagEnum.REFUNDING.getValue());
				this.updateById(order);
				orderRefundService.save(orderRefund);
			} else {
				// 自营商品直接退款
				order.setPayFlag(PayFlagEnum.REFUNDING.getValue());
				this.updateById(order);
				this.commonRefund(order);
			}
		}
	}

	@Resource
	private IRedIntegralService redIntegralService;

	@Transactional(rollbackFor = Exception.class)
	@Override
	public boolean payCommonSuccess(Order queryOrder, String payCode) {
		try {
			String outTradeNo = queryOrder.getOutTradeNo();
			if(!PayFlagEnum.WAIT_PAY.getValue().equals(queryOrder.getPayFlag())){
				log.info("订单号异常不能支付，订单号：" + outTradeNo + ",当前状态:" + queryOrder.getPayFlag());
				throw new ServiceException("订单号异常不能支付，订单号：" + outTradeNo + ",当前状态:" + queryOrder.getPayFlag());
			}
			//更新订单
			Order order = new Order();
			order.setId(queryOrder.getId());
			order.setPayFlag(2); //待发货
			//当订单类型为4的时候，直接设置为已完成
			if(queryOrder.getOrderType() == 3 || queryOrder.getOrderType() == 4 ){
				order.setPayFlag(4); //已完成
			}
			order.setOrderState(2);//已付款
			order.setPayTime(DateUtil.now()); //支付时间
			order.setPayCode(payCode); //支付方式
			this.setSettlementAccount(order);
			order.setOutTradeNo(queryOrder.getOutTradeNo());
			order.setUserAddressId(queryOrder.getUserAddressId());

			log.info("orderType ====> {}", queryOrder.getOrderType());

			if (OrderTypeEnum.LINKED.getValue().equals(queryOrder.getOrderType())) {
				// 更新用户联动模式订单
				Boolean linkedOrderResult = linkedOrderService.updateLinkedOrder(queryOrder);
				order.setLinkedResult(linkedOrderResult);
				// 返回结果为true则记录红包金额
				if (Boolean.TRUE.equals(linkedOrderResult)) {
					coRichEarnings(queryOrder);
				}
			}

			// 红包商城订单
			if (OrderTypeEnum.RE_ORDER.getValue().equals(queryOrder.getOrderType()) || OrderTypeEnum.RE_EXCHANGE.getValue().equals(queryOrder.getOrderType())
				|| OrderTypeEnum.MAKE.getValue().equals(queryOrder.getOrderType()) || OrderTypeEnum.PARTNER.getValue().equals(queryOrder.getOrderType()) ||
				(OrderTypeEnum.LINKED.getValue().equals(queryOrder.getOrderType()) && Boolean.FALSE.equals(order.getLinkedResult()))) {
				// 及时到账
				if (SettleTypeEnum.SELF_SUPPORT.getValue().equals(queryOrder.getSettleType())) {
					log.info("调用红包加速接口");
					// 调用红包加速接口
					redIntegralService.calculateIntegral(queryOrder.getOutTradeNo());
				}
				if (!OrderTypeEnum.PARTNER.getValue().equals(queryOrder.getOrderType())) {
					// 记录收益明细
					this.earningsDetail(queryOrder);
				}

				if (OrderTypeEnum.RE_EXCHANGE.getValue().equals(queryOrder.getOrderType()) || OrderTypeEnum.PARTNER.getValue().equals(queryOrder.getOrderType())) {
					// 修改用户积分信息
					exchangeModifyUserTicket(queryOrder);
				}
			}

			// 普通订单（绿色积分订单）
			if (OrderTypeEnum.NORMAL.getValue().equals(queryOrder.getOrderType()) && queryOrder.getPayFlag() == 1){
				if (SettleTypeEnum.SELF_SUPPORT.getValue().equals(queryOrder.getSettleType())) {
					// 修改用户积分信息
					exchangeModifyUserTicket(queryOrder);
				}
			}
			if (OrderTypeEnum.NORMAL.getValue().equals(queryOrder.getOrderType()) || OrderTypeEnum.RE_ORDER.getValue().equals(queryOrder.getOrderType()) || OrderTypeEnum.LINKED.getValue().equals(queryOrder.getOrderType())
				|| OrderTypeEnum.RE_EXCHANGE.getValue().equals(queryOrder.getOrderType()) || OrderTypeEnum.MAKE.getValue().equals(queryOrder.getOrderType()) || OrderTypeEnum.PARTNER.getValue().equals(queryOrder.getOrderType())) {
				this.updateGoodsInfo(order);
			}

			// 双十一商品赠送1:1绿色积分
			OrderGoods orderGoods = orderGoodsService.lambdaQuery().eq(OrderGoods::getOutTradeNo, outTradeNo).one();
			if (orderGoods != null && YesOrNoEnum.YES.getValue().equals(orderGoods.getIsActivity()) && SettleTypeEnum.SELF_SUPPORT.getValue().equals(queryOrder.getSettleType())) {
				this.doubleElevenIntegral(order);
			}

			int i = baseMapper.update(order,Wrappers.<Order>lambdaQuery().eq(Order::getOutTradeNo, outTradeNo));
			if (i != 1){
				logger.error("订单异常", "订单回调失败！订单号：" + outTradeNo);
				return false;
			}

			return true;
		} catch (Exception e) {
			log.error("订单支付成功回调异常, 订单号:" + queryOrder.getOutTradeNo() + e.getMessage(), e);
			logger.error("订单支付成功回调异常", queryOrder.getOutTradeNo() + e.getMessage());
			orderExceptionService.saveExceptionOrder(queryOrder, e.getMessage(), payCode, e);
			return false;
		}
	}

	private void setSettlementAccount(Order order) {
		if (CommonConstant.ALI_PAY.equals(order.getPayCode())) {
			order.setSettlementAccount(zhzxAliService.getPayeeUserId());
		}
		if (CommonConstant.WX_PAY.equals(order.getPayCode())) {
			order.setSettlementAccount(wxPayV3Bean.getMchId());
		}
	}

	/**
	 * 双十一赠送绿色积分
	 */
	private void doubleElevenIntegral(Order order) {
		UserApp userApp = userAppMapper.selectOne(Wrappers.lambdaQuery(UserApp.class).eq(UserApp::getUserId, order.getUserId()));
		// 积分记录
		Double ticket = NumberUtil.add(userApp.getTicket(), order.getTotalFee());
		UserTicket userTicket = new UserTicket();
		userTicket.setDataSrc(12);//双十一赠送
		userTicket.setServesRatio(0D);
		userTicket.setTicket(order.getTotalFee());
		userTicket.setUserId(userApp.getUserId());
		userTicket.setUserTicket(userApp.getTicket());
		userTicket.setTicketType(TicketTypeEnum.INCOME.getValue());//收入
		userTicket.setCreateTime(DateUtil.now());
		userTicket.setDataId(order.getId()); //订单id
		userTicket.setDataRemarks("双十一活动赠送");
		userTicket.setLastTicket(ticket);
		userTicketService.save(userTicket);

		userApp.setTicket(userTicket.getLastTicket());
		userAppMapper.updateById(userApp);
	}

	/**
	 * 积分兑换修改用户积分
	 */
	private void exchangeModifyUserTicket(Order order) {
		UserApp userApp = userAppMapper.selectOne(Wrappers.lambdaQuery(UserApp.class).eq(UserApp::getUserId, order.getUserId()));
		if (OrderTypeEnum.RE_EXCHANGE.getValue().equals(order.getOrderType())) {
			// 积分记录
			UserTicket userTicket = new UserTicket();
			userTicket.setDataSrc(8);//积分兑换
			userTicket.setServesRatio(0D);
			userTicket.setTicket(order.getNeedIntegral());
			userTicket.setUserId(userApp.getUserId());
			userTicket.setUserTicket(userApp.getTicket());
			userTicket.setTicketType(TicketTypeEnum.EXPENDITURE.getValue());
			userTicket.setCreateTime(DateUtil.now());
			userTicket.setDataId(order.getId());
			userTicket.setDataRemarks("积分兑换扣除");
			userTicket.setLastTicket(userApp.getTicket() - order.getNeedIntegral()); //最后酒票数
			userTicketService.save(userTicket);

			// 减掉用户积分
			userApp.setTicket(userTicket.getLastTicket());
			userAppMapper.updateById(userApp);
		}

		if (OrderTypeEnum.PARTNER.getValue().equals(order.getOrderType()) && order.getNeedIntegral() != null && order.getNeedIntegral() > 0) {
			// 积分记录
			UserTicket userTicket = new UserTicket();
			userTicket.setDataSrc(8);//积分兑换
			userTicket.setServesRatio(0D);
			userTicket.setTicket(order.getNeedIntegral());
			userTicket.setUserId(userApp.getUserId());
			userTicket.setUserTicket(userApp.getTicket());
			userTicket.setTicketType(TicketTypeEnum.EXPENDITURE.getValue());
			userTicket.setCreateTime(DateUtil.now());
			userTicket.setDataId(order.getId());
			userTicket.setDataRemarks("积分购买扣除");
			userTicket.setLastTicket(userApp.getTicket() - order.getNeedIntegral()); //最后酒票数
			userTicketService.save(userTicket);

			// 减掉用户积分
			userApp.setTicket(userTicket.getLastTicket());
			userAppMapper.updateById(userApp);
		}

		if (OrderTypeEnum.NORMAL.getValue().equals(order.getOrderType())) {
			Double ticket = NumberUtil.add(userApp.getTicket(), order.getTotalTicket());
			userApp.setTicket(ticket);
			userApp.updateById();
			UserTicket userTicket = new UserTicket();
			userTicket.setDataSrc(1);//1.订单
			userTicket.setServesRatio(0D);
			userTicket.setTicket(order.getTotalTicket());
			userTicket.setUserId(userApp.getUserId());
			userTicket.setUserTicket(userApp.getTicket());
			userTicket.setTicketType(TicketTypeEnum.INCOME.getValue());//收入
			userTicket.setCreateTime(DateUtil.now());
			userTicket.setDataId(order.getId()); //订单号
			userTicket.setDataRemarks("赠送积分");
			userTicket.setLastTicket(ticket); //最后酒票数
			userTicketService.save(userTicket);
		}

	}

	/**
	 * 记录收益明细
	 */
	private void earningsDetail(Order queryOrder) {
		OrderGoods orderGoods = orderGoodsService.lambdaQuery().eq(OrderGoods::getOutTradeNo, queryOrder.getOutTradeNo()).one();
		GoodsInfo goodsInfo = goodsInfoService.getById(orderGoods.getGoodsId());
		//1、查询最新的红包配置
		RedSetting redSetting = redSettingService.getFirst();
		EarningsDetail earningsDetail = new EarningsDetail();
		// 计算收益金额
		if (YesOrNoEnum.YES.getValue().equals(goodsInfo.getIsSpecial())) {
			// 特殊商品取商品自定义比例
			earningsDetail.setAmount(BigDecimal.valueOf(queryOrder.getTotalFee()).multiply(goodsInfo.getRedEnvRatio()).setScale(2, BigDecimal.ROUND_FLOOR));
		} else {
			earningsDetail.setAmount(BigDecimal.valueOf(queryOrder.getTotalFee()).multiply(redSetting.getProportion()).setScale(2, BigDecimal.ROUND_FLOOR));
		}
		earningsDetail.setOrderNo(queryOrder.getOutTradeNo());
		earningsDetail.setOrderId(queryOrder.getId());
		earningsDetail.setState(1);
		earningsDetailService.save(earningsDetail);
	}

	/**
	 * 记录共富收益
	 */
	private void coRichEarnings(Order queryOrder) {
		String rate = ParamCache.getValueDefault(CommonConstant.LINKED_RED_AMOUNT, "79");
		EarningsDetail earningsDetail = new EarningsDetail();
		earningsDetail.setAmount(new BigDecimal(rate));
		earningsDetail.setOrderNo(queryOrder.getOutTradeNo());
		earningsDetail.setOrderId(queryOrder.getId());
		earningsDetail.setState(1);
		earningsDetailService.save(earningsDetail);
	}

	/**
	 * 更新商品数据
	 */
	private void updateGoodsInfo(Order order) {
		List<OrderGoods> orderGoods = orderGoodsService.lambdaQuery().eq(OrderGoods::getOutTradeNo, order.getOutTradeNo()).list();
		for (OrderGoods orderGood : orderGoods) {
			GoodsSpec spec = goodsSpecService.getById(orderGood.getGoodsSpecIds());
			GoodsInfo goodsInfo = goodsInfoService.getById(orderGood.getGoodsId());
			if (spec == null) {
				log.info("商品规格不存在，订单号[{}]", order.getOutTradeNo());
				throw new ServiceException("商品规格不存在，请刷新后重试");
			}
			// 他营商品调用三方下单接口
			if (BusinessTypeEnum.NONE_SELF_SUPPORT.getValue().equals(orderGood.getBusinessType())) {
				int num = orderGood.getGoodsNum();
				// 是否为套装规格，调用第三方的购买数=用户购买数*套装商品数量
				if (YesOrNoEnum.YES.getValue().equals(orderGood.getIsPackage())) {
					num = orderGood.getGoodsNum() * orderGood.getPackageNum();
				}
				// 胜天半子调用下单接口
				if (GoodsSourceEnum.STBZ.getValue().equals(goodsInfo.getSource())) {
					JSONObject result = supplyChainService.order(order.getOutTradeNo(), spec.getOptionId(), num, order.getUserAddressId());
					Integer code = result.getInteger("code");
					if (code != 1) {
						bladeLogger.error("供应链下单失败","result:" + result.toJSONString() + ",订单号：" + order.getOutTradeNo());
					} else {
						order.setThirdOrderState("2");
					}
				} else if (GoodsSourceEnum.WPS.getValue().equals(goodsInfo.getSource())) {
					// 唯品尚调用订单确认接口
					weiPinShangService.payCallBack(order.getOutTradeNo(), "4");
				} else {
					throw new ServiceException("商品来源属性错误");
				}
			}
			// 更新商品数据
			if (goodsInfo != null) {
				goodsInfo.setTotalStock(goodsInfo.getTotalStock() - orderGood.getGoodsNum());
				goodsInfo.setTotalSales(goodsInfo.getTotalSales() + orderGood.getGoodsNum());
				goodsInfoService.updateById(goodsInfo);
				// 更新规格数据
				spec.setStock(spec.getStock() - orderGood.getGoodsNum());
				spec.setSale(spec.getSale() + orderGood.getGoodsNum());
				goodsSpecService.updateById(spec);
			}
		}
	}
	@Resource
	private BladeLogger bladeLogger;

	@Override
	public boolean payConsignSuccess(Order queryOrder) {
		String outTradeNo = queryOrder.getOutTradeNo();
		//寄售订单
		if(queryOrder.getOrderType() == 3){
			//选择寄售方式为寄售则生成新的寄售
			OrderConsign query = orderConsignMapper.selectOne(Wrappers.<OrderConsign>lambdaQuery().eq(OrderConsign::getOutTradeNo, outTradeNo));
			if(query == null ){
				logger.error("订单异常", "无订单数据！订单号：" + outTradeNo);
				return false;
			}
			//查询所抢的寄售订单
			Consign queryConsign = consignMapper.selectById(query.getConsignId());
			if(ObjectUtil.isNull(queryConsign)){ //后台的数据为0或者寄售的
				logger.error("订单异常", "寄售无数据！订单号：" + outTradeNo);
				return false;
			}
			//新建寄售单
			ConsignConfig consignConfig = consignConfigMapper.selectById(queryConsign.getConsignConfigId());
			Date startDate = cn.hutool.core.date.DateUtil.offsetDay(consignConfig.getStartTime(), 1);
			Date endDate = cn.hutool.core.date.DateUtil.offsetDay(consignConfig.getEndTime(), 1);
			Consign consign = new Consign();
			consign.setOrderId(queryOrder.getId()); //保存来源的订单ID
			consign.setConsignFor(2);//寄售订单
			consign.setConsignConfigId(queryConsign.getConsignConfigId());
			consign.setConsignDays(query.getConsignDays());
			consign.setConsignUserId(query.getPayUserId());//支付人变为寄售人
			consign.setConsignType(0);//无
			consign.setTotalNum(1);
			consign.setNowStock(1);
			consign.setIsDelivery(1); //是否交割 否
			consign.setIsSale(2); //是否上架 是（自动上架）
			consign.setDeliveryPrice(0d);//默认为0，需要后台设置交割价
			consign.setGoodsId(query.getGoodsId());
			consign.setGoodsName(query.getGoodsName());
			consign.setTitle(query.getGoodsName());
			Double consignPrice = ConsignWrapper.build().consignPrice(query.getConsignPrice(), query.getConsignDays());
			consign.setObtainPrice(query.getConsignPrice()); //取得价为抢购别人的价格
			consign.setConsignPrice(consignPrice);//抢购价
			consign.setStartTime(startDate);
			consign.setEndTime(endDate);
			consign.setSaleDays(0);
			int j = consignMapper.insert(consign);
			if(j != 1){
				logger.error("订单异常", "订单变为寄售失败！订单号：" + outTradeNo);
				throw new ServiceException("亲！请重新抢单");
			}
			//更新寄售状态为已售出
			if(queryConsign.getNowStock() == 0){
				Consign updateConsign = new Consign();
				updateConsign.setId(query.getConsignId());
				updateConsign.setConsignState(3);//已售出
				int k =	consignMapper.updateById(updateConsign);
				if(k != 1){
					logger.error("订单异常", "更新寄售状态！订单号：" + outTradeNo);
					throw new ServiceException("亲！请重新抢单");
				}
			}
		}
		return true;
	}


	@Override
	public boolean orderEnd(Long orderId, Long userId) {
		Order order = baseMapper.selectById(orderId);
		//必须是支付状态为待收货
		if (!order.getPayFlag().equals(PayFlagEnum.WAIT_RECEIVE.getValue())){
			throw new ServiceException("请等待商家发货!");
		}
		if (SettleTypeEnum.NONE_SELF_SUPPORT.getValue().equals(order.getSettleType())) {
			if (OrderTypeEnum.RE_ORDER.getValue().equals(order.getOrderType()) || OrderTypeEnum.RE_EXCHANGE.getValue().equals(order.getOrderType())
				|| OrderTypeEnum.MAKE.getValue().equals(order.getOrderType()) || OrderTypeEnum.PARTNER.getValue().equals(order.getOrderType())
				|| OrderTypeEnum.LINKED.getValue().equals(order.getOrderType())) {
				// 调用红包加速接口
				redIntegralService.calculateIntegral(order.getOutTradeNo());
			}
			if (OrderTypeEnum.NORMAL.getValue().equals(order.getOrderType())) {
				exchangeModifyUserTicket(order);
			}
			// 双十一商品赠送1:1绿色积分
			OrderGoods orderGoods = orderGoodsService.lambdaQuery().eq(OrderGoods::getOutTradeNo, order.getOutTradeNo()).one();
			if (YesOrNoEnum.YES.getValue().equals(orderGoods.getIsActivity())) {
				this.doubleElevenIntegral(order);
			}
		} else {
			log.info("订单[{}]不满足条件，不赠送积分", order.getOutTradeNo());
		}

		List<OrderGoods> list = orderGoodsService.lambdaQuery().eq(OrderGoods::getOutTradeNo, order.getOutTradeNo()).list();
		if (CollUtil.isNotEmpty(list)) {
			GoodsInfo goodsInfo = goodsInfoService.getById(list.get(0).getGoodsId());
			if (goodsInfo != null && BusinessTypeEnum.NONE_SELF_SUPPORT.getValue().equals(goodsInfo.getBusinessType())
				&& GoodsSourceEnum.WPS.getValue().equals(goodsInfo.getSource())) {
				// 调用唯品尚确认收货接口
				weiPinShangService.receivedOrder(order.getOutTradeNo());
			}
		}

		Order param = new Order();
		param.setId(orderId);
		param.setPayFlag(PayFlagEnum.COMPLETED.getValue()); //已完成
		param.setRecordTime(DateUtil.now());
		int i = baseMapper.updateById(param);
		return i == 1;
	}

}
