/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.modules.platform.service.impl;

import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springblade.common.enums.TicketTypeEnum;
import org.springblade.core.log.exception.ServiceException;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.modules.platform.dto.AddUserTicketDTO;
import org.springblade.modules.platform.entity.UserTicket;
import org.springblade.modules.platform.mapper.UserTicketMapper;
import org.springblade.modules.platform.service.IUserTicketService;
import org.springblade.modules.platform.vo.EarningsTicketDetailVO;
import org.springblade.modules.platform.vo.EarningsTicketStatisVO;
import org.springblade.modules.platform.vo.IntegralInfoVO;
import org.springblade.modules.platform.vo.UserTicketVO;
import org.springblade.modules.redenvelopemall.mapper.RedEnvelopeReceiveLogMapper;
import org.springblade.modules.redenvelopemall.mapper.RedIntegralMapper;
import org.springblade.modules.system.entity.User;
import org.springblade.modules.system.entity.UserApp;
import org.springblade.modules.system.mapper.UserMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 用户酒票表 服务实现类
 *
 * @author BladeX
 * @since 2022-02-13
 */
@Service
public class UserTicketServiceImpl extends ServiceImpl<UserTicketMapper, UserTicket> implements IUserTicketService {

	@Autowired
	private UserAppServiceImpl userAppService;
	@Resource
	private UserMapper userMapper;
	@Resource
	private RedIntegralMapper redIntegralMapper;
	@Resource
	private RedEnvelopeReceiveLogMapper redEnvelopeReceiveLogMapper;

	@Override
	public IPage<UserTicketVO> selectUserTicketPage(IPage<UserTicketVO> page, UserTicketVO userTicket) {
		return page.setRecords(baseMapper.selectUserTicketPage(page, userTicket));
	}

	@Transactional(rollbackFor = Exception.class)
	@Override
	public void addUserScore(AddUserTicketDTO addUserTicketDTO) {
		User user = userMapper.selectById(addUserTicketDTO.getUserId());
		if (user == null) {
			throw new ServiceException("用户不存在");
		}
		UserApp userApp = userAppService.getBaseMapper().selectOne(Wrappers.<UserApp>lambdaQuery()
			.eq(UserApp::getUserId, addUserTicketDTO.getUserId()));
		if (userApp == null) {
			throw new ServiceException("用户积分信息不存在");
		}
		// 添加积分记录
		UserTicket ticket = new UserTicket();
		ticket.setUserId(addUserTicketDTO.getUserId());
		ticket.setDataSrc(addUserTicketDTO.getDataSrc());//后台录入
		ticket.setTicket(addUserTicketDTO.getScore());
		ticket.setDataRemarks(addUserTicketDTO.getRemark());
		ticket.setTicketType(TicketTypeEnum.INCOME.getValue());//收入
		ticket.setLastTicket(userApp.getTicket());
		ticket.setUserTicket(userApp.getTicket() + addUserTicketDTO.getScore());
		ticket.setCreateTime(new Date());
		ticket.setDataId(AuthUtil.getUserId());
		this.getBaseMapper().insert(ticket);

		// 修改用户积分
		UserApp update = new UserApp();
		update.setId(userApp.getId());
		update.setTicket(ticket.getUserTicket());
		userAppService.updateById(update);
	}

	@Transactional(rollbackFor = Exception.class)
	@Override
	public void subUserScore(AddUserTicketDTO addUserTicketDTO) {
		User user = userMapper.selectById(addUserTicketDTO.getUserId());
		if (user == null) {
			throw new ServiceException("用户不存在");
		}
		UserApp userApp = userAppService.getBaseMapper().selectOne(Wrappers.<UserApp>lambdaQuery()
			.eq(UserApp::getUserId, addUserTicketDTO.getUserId()));
		if (userApp == null) {
			throw new ServiceException("用户积分信息不存在");
		}
		// 添加积分记录
		UserTicket ticket = new UserTicket();
		ticket.setUserId(addUserTicketDTO.getUserId());
		ticket.setDataSrc(addUserTicketDTO.getDataSrc());
		ticket.setTicket(addUserTicketDTO.getScore());
		ticket.setDataRemarks(addUserTicketDTO.getRemark());
		ticket.setTicketType(TicketTypeEnum.EXPENDITURE.getValue());
		ticket.setLastTicket(userApp.getTicket());
		ticket.setUserTicket(userApp.getTicket() - addUserTicketDTO.getScore());
		ticket.setCreateTime(new Date());
		ticket.setDataId(AuthUtil.getUserId());
		this.getBaseMapper().insert(ticket);
		// 修改用户积分
		UserApp update = new UserApp();
		update.setId(userApp.getId());
		update.setTicket(ticket.getUserTicket());
		userAppService.updateById(update);
	}

	/**
	 * 积分变更
	 * @param userId 用户id
	 * @param score 积分
	 * @param src 来源
	 * @param remake 备注
	 * @param ticketType 插入类型，1:收入 2：支出
	 */
	@Override
	public void addScoreBySrc2Data(Long userId, BigDecimal score, Integer src, String remake, TicketTypeEnum ticketType, Long dataId){
		UserApp userApp = userAppService.getBaseMapper().selectOne(Wrappers.<UserApp>lambdaQuery()
			.eq(UserApp::getUserId, userId));
		// 添加积分记录
		UserTicket ticket = new UserTicket();
		ticket.setUserId(userId);
		ticket.setDataSrc(src);
		ticket.setTicket(score.doubleValue());
		ticket.setDataRemarks(remake);
		ticket.setTicketType(ticketType.getValue());//收入
		BigDecimal changeTicket = new BigDecimal(userApp.getTicket());
		//通过添加类型判断收入还是支出
		if(ticketType.getValue() == 1){
			changeTicket = changeTicket.add(score);
		}else {
			changeTicket = changeTicket.subtract(score);
		}
		ticket.setUserTicket(userApp.getTicket());
		ticket.setLastTicket(changeTicket.doubleValue());
		ticket.setCreateTime(new Date());
		ticket.setDataId(dataId);
		this.getBaseMapper().insert(ticket);
		// 修改用户积分
		UserApp update = new UserApp();
		update.setId(userApp.getId());
		update.setTicket(changeTicket.doubleValue());
		userAppService.updateById(update);
	}

	/**
	 * 积分变更
	 * @param userId 用户id
	 * @param score 积分
	 * @param src 来源
	 * @param remake 备注
	 * @param ticketType 插入类型，1:收入 2：支出
	 */
	@Override
	public void addScoreBySrc(Long userId, BigDecimal score, Integer src, String remake, TicketTypeEnum ticketType){
		UserApp userApp = userAppService.getBaseMapper().selectOne(Wrappers.<UserApp>lambdaQuery()
			.eq(UserApp::getUserId, userId));
		// 添加积分记录
		UserTicket ticket = new UserTicket();
		ticket.setUserId(userId);
		ticket.setDataSrc(src);
		ticket.setTicket(score.doubleValue());
		ticket.setDataRemarks(remake);
		ticket.setTicketType(ticketType.getValue());//收入
		BigDecimal changeTicket = new BigDecimal(userApp.getTicket());
		//通过添加类型判断收入还是支出
		if(ticketType.getValue() == 1){
			changeTicket = changeTicket.add(score);
		}else {
			changeTicket = changeTicket.subtract(score);
		}
		ticket.setUserTicket(userApp.getTicket());
		ticket.setLastTicket(changeTicket.doubleValue());
		ticket.setCreateTime(new Date());
		ticket.setDataId(AuthUtil.getUserId());
		this.getBaseMapper().insert(ticket);
		// 修改用户积分
		UserApp update = new UserApp();
		update.setId(userApp.getId());
		update.setTicket(changeTicket.doubleValue());
		userAppService.updateById(update);
	}

	@Override
	public EarningsTicketStatisVO earningsTicketStatic(Long userId, Integer type) {
		EarningsTicketStatisVO vo = new EarningsTicketStatisVO();
		LambdaQueryWrapper<UserTicket> eq = Wrappers.lambdaQuery(UserTicket.class)
			.eq(UserTicket::getUserId, userId).eq(UserTicket::getDataSrc, type);
		QueryWrapper<UserTicket> userTicketQueryWrapper = new QueryWrapper<>();
		userTicketQueryWrapper.select("sum(ticket) as allNum");
		userTicketQueryWrapper.eq("user_id", userId);
		userTicketQueryWrapper.eq("data_src", type);
		UserTicket userTicket = this.baseMapper.selectOne(userTicketQueryWrapper);
//		Integer integer = this.getBaseMapper().selectCount(Wrappers.lambdaQuery(UserTicket.class)
//			.eq(UserTicket::getUserId, userId).eq(UserTicket::getDataSrc, type));
		if(ObjectUtil.isNotNull(userTicket)){
			vo.setTotalTicket(userTicket.getAllNum().setScale(2, BigDecimal.ROUND_DOWN).doubleValue());
		}else{
			vo.setTotalTicket(0d);
		}

		return vo;
	}

	@Override
	public EarningsTicketDetailVO earningsTicketDetail(Long userId, Integer type) {

		return null;
	}

	public static void main(String[] args) {
		System.out.println(DateUtil.formatDate(DateUtil.beginOfMonth(new Date())));
	}

	@Override
	public IPage<IntegralInfoVO> info(IPage<User> page, String name, Integer type) {
//		LambdaQueryChainWrapper<User> wrapper = userService.lambdaQuery();
		String currMonthBegin = DateUtil.formatDate(DateUtil.beginOfMonth(new Date()));
		QueryWrapper<User> wrapper = new QueryWrapper<>();
		if (StrUtil.isNotBlank(name)) {
			wrapper.eq("real_name", name);
		}
		wrapper.eq("user_type", 2);
		IPage<User> userIPage = userMapper.selectPage(page, wrapper);
		List<IntegralInfoVO> list = new ArrayList<>();
		for (User user : userIPage.getRecords()) {
			IntegralInfoVO info = null;
			if (type == 1) {
				info = this.baseMapper.info(user.getId(), currMonthBegin);
			}
			if (type == 2) {
				info = redIntegralMapper.info(user.getId(), currMonthBegin);
			}
			if (type == 3) {
				info = redEnvelopeReceiveLogMapper.info(user.getId(), currMonthBegin, 0);
			}
			if (type == 4) {
				info = redEnvelopeReceiveLogMapper.info(user.getId(), currMonthBegin, 1);
			}
			if (info == null) {
				info = new IntegralInfoVO();
			}
			info.setUserId(user.getId());
			info.setPhone(user.getPhone());
			info.setUserName(user.getRealName());
			info.setAccount(user.getAccount());
			info.setTypeDesc(this.getTypeDesc(type));
			list.add(info);
		}
		IPage<IntegralInfoVO> infoVOIPage = new Page<>();
		infoVOIPage.setRecords(list);
		infoVOIPage.setTotal(userIPage.getTotal());
		return infoVOIPage;
	}

	private String getTypeDesc(Integer type) {
		if (type == 1) {
			return "绿色积分";
		} else if(type == 2) {
			return "红色积分";
		} else if (type == 3) {
			return "抢红包";
		} else {
			return "分红型红包";
		}
	}
}
