/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.modules.redenvelopemall.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import javax.validation.Valid;

import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.sms.model.SmsCode;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.modules.finance.entity.Order;
import org.springblade.modules.finance.service.IOrderService;
import org.springblade.modules.platform.dto.AddUserTicketDTO;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.annotation.RequestParam;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.modules.redenvelopemall.entity.RedIntegral;
import org.springblade.modules.redenvelopemall.vo.RedIntegralVO;
import org.springblade.modules.redenvelopemall.wrapper.RedIntegralWrapper;
import org.springblade.modules.redenvelopemall.service.IRedIntegralService;
import org.springblade.core.boot.ctrl.BladeController;

/**
 * 红色积分变更记录表 控制器
 *
 * @author BladeX
 * @since 2022-05-25
 */
@RestController
@AllArgsConstructor
@RequestMapping("redenvelopemall/redintegral")
@Api(value = "红色积分变更记录表", tags = "红色积分变更记录表接口")
public class RedIntegralController extends BladeController {

	private final IRedIntegralService redIntegralService;
	private final IOrderService orderService;

	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情", notes = "传入redIntegral")
	public R<RedIntegralVO> detail(RedIntegral redIntegral) {
		RedIntegral detail = redIntegralService.getOne(Condition.getQueryWrapper(redIntegral));
		return R.data(RedIntegralWrapper.build().entityVO(detail));
	}

	/**
	 * 分页 红色积分变更记录表
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "分页", notes = "传入redIntegral")
	public R<IPage<RedIntegralVO>> list(RedIntegral redIntegral, Query query) {
		IPage<RedIntegral> pages = redIntegralService.page(Condition.getPage(query), Condition.getQueryWrapper(redIntegral));
		return R.data(RedIntegralWrapper.build().pageVO(pages));
	}


	/**
	 * 自定义分页 红色积分变更记录表
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "分页", notes = "传入redIntegral")
	public R<IPage<RedIntegralVO>> page(RedIntegralVO redIntegral, Query query) {
		IPage<RedIntegralVO> pages = redIntegralService.selectRedIntegralPage(Condition.getPage(query), redIntegral);
		return R.data(pages);
	}

	/**
	 * 新增 红色积分变更记录表
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "新增", notes = "传入redIntegral")
	public R save(@Valid @RequestBody RedIntegral redIntegral) {
		return R.status(redIntegralService.save(redIntegral));
	}

	/**
	 * 修改 红色积分变更记录表
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "修改", notes = "传入redIntegral")
	public R update(@Valid @RequestBody RedIntegral redIntegral) {
		return R.status(redIntegralService.updateById(redIntegral));
	}

	/**
	 * 新增或修改 红色积分变更记录表
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "新增或修改", notes = "传入redIntegral")
	public R submit(@Valid @RequestBody RedIntegral redIntegral) {
		return R.status(redIntegralService.saveOrUpdate(redIntegral));
	}

	@PostMapping("/addByOrder")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "根据订单添加积分")
	public R addByOrder(@ApiParam(value = "订单号", required = true) @RequestParam String orderNos) {
		for (String orderNo : Func.toStrList(orderNos)) {
			redIntegralService.calculateIntegral(orderNo);
		}
		return R.data("操作成功");
	}

	@PostMapping("/refundDeduct")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "根据订单扣除积分")
	public R refundDeduct(@ApiParam(value = "订单号", required = true) @RequestParam String orderNo) {
		Order order = orderService.lambdaQuery().eq(Order::getOutTradeNo, orderNo).one();
		if (order != null) {
			redIntegralService.refundDeduct(order);
			return R.success("ok");
		}
		return R.fail("订单数据不存在");
	}

	@PostMapping("/orderReturn")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "根据订单退还积分")
	public R orderReturn(@ApiParam(value = "订单号", required = true) @RequestParam String orderNo) {
		redIntegralService.orderReturn(orderNo);
		return R.success("ok");
	}

	@PostMapping("addUserScore")
	@ApiOperation(value = "添加用户积分", notes = "传入ids")
	public R addUserScore(@RequestBody AddUserTicketDTO addUserTicketDTO) {
		if (addUserTicketDTO.getScore() < 1) {
			return R.fail("积分数不能小于1");
		}
		addUserTicketDTO.setDataSrc(5);
		redIntegralService.addUserScore(addUserTicketDTO);
		return R.success("添加成功");
	}

}
