/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.modules.redenvelopemall.service.impl;

import cn.hutool.core.collection.ArrayIter;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.extern.slf4j.Slf4j;
import org.springblade.common.cache.ParamCache;
import org.springblade.common.constant.CommonConstant;
import org.springblade.common.enums.BusinessTypeEnum;
import org.springblade.common.enums.GoodsSourceEnum;
import org.springblade.common.enums.YesOrNoEnum;
import org.springblade.common.enums.ZoneTypeEnum;
import org.springblade.core.log.exception.ServiceException;
import org.springblade.modules.redenvelopemall.entity.ActivitiesGoods;
import org.springblade.modules.redenvelopemall.entity.GoodsImg;
import org.springblade.modules.redenvelopemall.entity.GoodsInfo;
import org.springblade.modules.redenvelopemall.entity.GoodsSpec;
import org.springblade.modules.redenvelopemall.service.IActivitiesGoodsService;
import org.springblade.modules.redenvelopemall.service.IWeiPinShangService;
import org.springblade.modules.redenvelopemall.vo.GoodsInfoVO;
import org.springblade.modules.redenvelopemall.mapper.GoodsInfoMapper;
import org.springblade.modules.redenvelopemall.service.IGoodsInfoService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.support.atomic.RedisAtomicLong;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 商品表 服务实现类
 *
 * @author BladeX
 * @since 2022-05-07
 */
@Slf4j
@Service
public class GoodsInfoServiceImpl extends ServiceImpl<GoodsInfoMapper, GoodsInfo> implements IGoodsInfoService {

	@Resource
	private GoodsImgServiceImpl goodsImgService;
	@Resource
	private RedisTemplate redisTemplate;
	@Resource
	private GoodsSpecServiceImpl goodsSpecService;
	@Resource
	private SupplyChainServiceImpl supplyChainService;
	@Resource
	private IWeiPinShangService weiPinShangService;
	@Resource
	private IActivitiesGoodsService activitiesGoodsService;

	private final static String GOODS_NO_PREFIX = "re:goods:info:no";

	@Override
	public GoodsInfo get(Serializable id) {
		return this.baseMapper.get(id);
	}

	@Override
	public IPage<GoodsInfoVO> selectGoodsInfoPage(IPage<GoodsInfoVO> page, GoodsInfoVO goodsInfo) {
		return page.setRecords(baseMapper.selectGoodsInfoPage(page, goodsInfo));
	}

	@Transactional(rollbackFor = Exception.class)
	@Override
	public void update(@Valid GoodsInfo goodsInfo) {
		GoodsInfo record = this.getById(goodsInfo.getId());
		Assert.notNull(record, "商品信息不存在");
		this.updateById(goodsInfo);

		// 删除老的图片数据
		LambdaQueryWrapper<GoodsImg> wrapper = Wrappers.lambdaQuery(GoodsImg.class).eq(GoodsImg::getGoodsId, goodsInfo.getId());
		List<GoodsImg> list = goodsImgService.list(wrapper);
		if (CollUtil.isNotEmpty(list)) {
			goodsImgService.remove(wrapper);
		}

		// 保存商品图片数据
		List<GoodsImg> goodsImgList = new ArrayList<>();
		List<String> detailImgUrlList = goodsInfo.getDetailImgUrlList();
		if (CollUtil.isNotEmpty(detailImgUrlList)) {
			for (int i = 0; i < detailImgUrlList.size(); i++) {
				String url = detailImgUrlList.get(i);
				goodsImgList.add(new GoodsImg(goodsInfo.getId(), url, 2, 1, i + 1));
			}
		}
		List<String> slideshowList = goodsInfo.getSlideshowList();
		if (StrUtil.isNotBlank(goodsInfo.getVideoUrl())) {
			goodsImgList.add(new GoodsImg(goodsInfo.getId(), goodsInfo.getVideoUrl(), 1, 2, 1));
		}
		if (CollUtil.isNotEmpty(slideshowList)) {
			for (int i = 0; i < slideshowList.size(); i++) {
				String url = slideshowList.get(i);
				goodsImgList.add(new GoodsImg(goodsInfo.getId(), url, 1, 1, i + 2));
			}
		}
		if (CollUtil.isNotEmpty(goodsImgList)) {
			goodsImgService.saveBatch(goodsImgList);
		}
	}

	@Transactional(rollbackFor = Exception.class)
	@Override
	public void submit(@Valid GoodsInfo goodsInfo) {
		goodsInfo.setId(null);
		goodsInfo.setGoodsNo(generateGoodsNo());
		this.save(goodsInfo);
		// 保存商品图片数据
		List<GoodsImg> goodsImgList = new ArrayList<>();
		List<String> detailImgUrlList = goodsInfo.getDetailImgUrlList();
		if (CollUtil.isNotEmpty(detailImgUrlList)) {
			for (int i = 0; i < detailImgUrlList.size(); i++) {
				String url = detailImgUrlList.get(i);
				goodsImgList.add(new GoodsImg(goodsInfo.getId(), url, 2, 1, i + 1));
			}
		}
		List<String> slideshowList = goodsInfo.getSlideshowList();
		if (StrUtil.isNotBlank(goodsInfo.getVideoUrl())) {
			goodsImgList.add(new GoodsImg(goodsInfo.getId(), goodsInfo.getVideoUrl(), 1, 2, 1));
		}
		if (CollUtil.isNotEmpty(slideshowList)) {
			for (int i = 0; i < slideshowList.size(); i++) {
				String url = slideshowList.get(i);
				goodsImgList.add(new GoodsImg(goodsInfo.getId(), url, 1, 1, i + 2));
			}
		}
		if (CollUtil.isNotEmpty(goodsImgList)) {
			goodsImgService.saveBatch(goodsImgList);
		}
	}

	@Override
	public GoodsInfo detail(Long id) {
		GoodsInfo detail = this.getById(id);
		List<GoodsImg> list = goodsImgService.list(Wrappers.lambdaQuery(GoodsImg.class).eq(GoodsImg::getGoodsId, detail.getId()));
		if (CollUtil.isNotEmpty(list)) {
			List<String> slideshowList = new ArrayList<>();
			List<String> detailImgUrlList = new ArrayList<>();
			for (GoodsImg goodsImg : list) {
				if (goodsImg.getFileType() == 2) {
					detail.setVideoUrl(goodsImg.getUrl());
					continue;
				}
				if (goodsImg.getType() == 1) {
					slideshowList.add(goodsImg.getUrl());
				} else {
					detailImgUrlList.add(goodsImg.getUrl());
				}
			}
			detail.setDetailImgUrlList(detailImgUrlList);
			detail.setSlideshowList(slideshowList);
		}
		return detail;
	}

	@Override
	public GoodsInfo getByGoodsNo(String goodsNo) {
		return this.getOne(Wrappers.lambdaQuery(GoodsInfo.class).eq(GoodsInfo::getGoodsNo, goodsNo));
	}

	@Override
	public List<GoodsInfo> getByThirdIds(List<String> thirdIds, GoodsSourceEnum goodsSourceEnum) {
		if (CollUtil.isNotEmpty(thirdIds)) {
			return this.getBaseMapper().selectList(Wrappers.lambdaQuery(GoodsInfo.class).in(GoodsInfo::getThirdId, thirdIds).eq(GoodsInfo::getSource, goodsSourceEnum.getValue()));
		}
		return new ArrayList<>();
	}

	@Override
	public void modifyState(List<Long> ids, Integer state) {
		List<GoodsInfo> goodsInfos = this.getBaseMapper().selectList(Wrappers.lambdaQuery(GoodsInfo.class).in(GoodsInfo::getId, ids));
		if (CollUtil.isNotEmpty(goodsInfos)) {
			List<GoodsInfo> updateList = new ArrayList<>();
			for (GoodsInfo goodsInfo : goodsInfos) {
				GoodsInfo update = new GoodsInfo();
				update.setId(goodsInfo.getId());
				update.setIsSale(state);
				updateList.add(update);
			}
			this.updateBatchById(updateList);
		}
	}

	@Override
	public void modifyCategory(List<Long> ids, String categoryIds) {
		List<GoodsInfo> goodsInfos = this.getBaseMapper().selectList(Wrappers.lambdaQuery(GoodsInfo.class).in(GoodsInfo::getId, ids));
		if (CollUtil.isNotEmpty(goodsInfos)) {
			List<GoodsInfo> updateList = new ArrayList<>();
			for (GoodsInfo goodsInfo : goodsInfos) {
				GoodsInfo update = new GoodsInfo();
				update.setId(goodsInfo.getId());
				update.setGoodsCategoryId(categoryIds + "-");
				updateList.add(update);
			}
			this.updateBatchById(updateList);
		}
	}

	@Transactional(rollbackFor = Exception.class)
	@Override
	public void modifyGrossRate(List<Long> ids, Double grossRate) {
		List<GoodsInfo> goodsInfos = this.getBaseMapper().selectList(Wrappers.lambdaQuery(GoodsInfo.class).in(GoodsInfo::getId, ids));
		if (CollUtil.isNotEmpty(goodsInfos)) {
			List<GoodsInfo> updateList = new ArrayList<>();
			for (GoodsInfo goodsInfo : goodsInfos) {
				GoodsInfo update = new GoodsInfo();
				update.setId(goodsInfo.getId());
				BigDecimal salePrice = this.calSalePrice(new BigDecimal(grossRate), goodsInfo.getAgreementPrice());
				update.setSalePrice(salePrice);
				update.setGuidePrice(salePrice);
				update.setPromotionRate(new BigDecimal(grossRate));
				updateList.add(update);
				List<GoodsSpec> list = goodsSpecService.lambdaQuery().eq(GoodsSpec::getGoodsId, goodsInfo.getId()).list();
				for (GoodsSpec goodsSpec : list) {
					BigDecimal specSalePrice = this.calSalePrice(new BigDecimal(grossRate), goodsSpec.getAgreementPrice());
					goodsSpecService.lambdaUpdate().set(GoodsSpec::getSalePrice, specSalePrice).set(GoodsSpec::getGuidePrice, specSalePrice)
						.set(GoodsSpec::getPromotionRate, new BigDecimal(grossRate)).eq(GoodsSpec::getId, goodsSpec.getId()).update();
				}
			}
			this.updateBatchById(updateList);
		}
	}

	@Override
	public BigDecimal calPromotionRate(BigDecimal salePrice, BigDecimal agreementPrice) {
		// 售价红包比例
		String rate = ParamCache.getValueDefault(CommonConstant.SALE_PRICE_RED_ENVELOPE_RATE, "0.2");
		BigDecimal multiply = salePrice.multiply(new BigDecimal(rate));
		BigDecimal subtract = salePrice.subtract(multiply).subtract(agreementPrice);
		return subtract.divide(salePrice, 2, BigDecimal.ROUND_HALF_UP).multiply(new BigDecimal(100));
	}

	/**
	 * 计算售价
	 */
	@Override
	public BigDecimal calSalePrice(BigDecimal grossRate, BigDecimal agreementPrice) {
		grossRate = grossRate.divide(new BigDecimal(100));
		// 售价红包比例
		String rate = ParamCache.getValueDefault(CommonConstant.SALE_PRICE_RED_ENVELOPE_RATE, "0.2");
		BigDecimal denominator = new BigDecimal(1).subtract(new BigDecimal(rate)).subtract(grossRate);
		return agreementPrice.divide(denominator, 2, BigDecimal.ROUND_HALF_UP);
	}

	@Transactional(rollbackFor = Exception.class)
	@Override
	public void modifyZoneType(List<Long> ids, Integer zoneType, Date activityDate) {
		List<GoodsInfo> goodsInfos = this.getBaseMapper().selectList(Wrappers.lambdaQuery(GoodsInfo.class).in(GoodsInfo::getId, ids));
		if (CollUtil.isNotEmpty(goodsInfos)) {
			for (GoodsInfo goodsInfo : goodsInfos) {
				goodsInfo.setZoneType(zoneType);
				this.updateById(goodsInfo);
			}
		}
		if (ZoneTypeEnum.GREEN_INTEGRAL.getValue().equals(zoneType)) {
			for (GoodsInfo goodsInfo : goodsInfos) {
				List<GoodsSpec> specList = goodsSpecService.lambdaQuery().eq(GoodsSpec::getGoodsId, goodsInfo.getId()).list();
				for (GoodsSpec goodsSpec : specList) {
					goodsSpec.setObtainIntegral(goodsSpec.getSalePrice());
				}
				goodsSpecService.updateBatchById(specList);
			}
		}
	}

	@Transactional(rollbackFor = Exception.class)
	@Override
	public void syncSpec(List<Long> ids) {
		for (Long id : ids) {
			GoodsInfo goodsInfo = this.getById(id);
			goodsSpecService.deleteByGoodsIds(Collections.singletonList(goodsInfo.getId()));
			JSONObject detail = supplyChainService.detail(Collections.singletonList(goodsInfo.getThirdId()));
			JSONArray data = detail.getJSONArray("data");
			supplyChainService.saveGoodsSpecInfo(data.getJSONObject(0), goodsInfo, goodsInfo.getPromotionRate().doubleValue());
		}
	}

	@Override
	public List<GoodsInfo> activityGoods(IPage<Object> page, String activityDate) {
		return this.baseMapper.activityGoods(page, activityDate);
	}

	@Transactional(rollbackFor = Exception.class)
	@Override
	public void setActivityGoods(List<Long> ids, DateTime activityDate) {
		List<GoodsInfo> goodsInfos = this.getBaseMapper().selectList(Wrappers.lambdaQuery(GoodsInfo.class).in(GoodsInfo::getId, ids));
		List<ActivitiesGoods> list = new ArrayList<>();
		for (GoodsInfo goodsInfo : goodsInfos) {
			if (ZoneTypeEnum.CITY_PARTNER.getValue().equals(goodsInfo.getZoneType()) || ZoneTypeEnum.GREEN_INTEGRAL.getValue().equals(goodsInfo.getZoneType()) || ZoneTypeEnum.INTEGRAL_CONVERT.getValue().equals(goodsInfo.getZoneType())) {
				throw new ServiceException("积分兑换专区、绿色积分专区、城市合伙人专区的商品暂不支持添加到活动中");
			}
			ActivitiesGoods record = activitiesGoodsService.lambdaQuery().eq(ActivitiesGoods::getGoodsInfoId, goodsInfo.getId()).one();
			if (record != null && !record.getActivityDate().equals(activityDate)) {
				activitiesGoodsService.removeById(record.getId());
				ActivitiesGoods activitiesGoods = new ActivitiesGoods();
				activitiesGoods.setGoodsInfoId(goodsInfo.getId());
				activitiesGoods.setActivityDate(activityDate);
				activitiesGoods.setStartTime(DateUtil.beginOfDay(activityDate));
				activitiesGoods.setEndTime(DateUtil.endOfDay(activityDate).offset(DateField.MILLISECOND, -999));
				list.add(activitiesGoods);
			}
			if (record == null){
				ActivitiesGoods activitiesGoods = new ActivitiesGoods();
				activitiesGoods.setGoodsInfoId(goodsInfo.getId());
				activitiesGoods.setActivityDate(activityDate);
				activitiesGoods.setStartTime(DateUtil.beginOfDay(activityDate));
				activitiesGoods.setEndTime(DateUtil.endOfDay(activityDate).offset(DateField.MILLISECOND, -999));
				list.add(activitiesGoods);
			}
		}
		activitiesGoodsService.saveBatch(list);
		this.lambdaUpdate().set(GoodsInfo::getIsActivity, YesOrNoEnum.YES.getValue()).in(GoodsInfo::getId, ids).update();
	}

	@Override
	public void syncFull(List<Long> ids, boolean isUpdateSalePrice) {
		List<GoodsInfo> list = this.lambdaQuery().in(GoodsInfo::getId, ids).eq(GoodsInfo::getBusinessType, BusinessTypeEnum.NONE_SELF_SUPPORT.getValue()).list();
		// 胜天半子商品
		List<GoodsInfo> stbzList = list.stream().filter(e -> e.getSource().equals(GoodsSourceEnum.STBZ.getValue())).collect(Collectors.toList());
		// 唯品尚商品
		List<GoodsInfo> wpsList = list.stream().filter(e -> e.getSource().equals(GoodsSourceEnum.WPS.getValue())).collect(Collectors.toList());
		if (CollUtil.isNotEmpty(wpsList)) {
			weiPinShangService.syncGoodsInfo(wpsList.stream().map(GoodsInfo::getThirdId).collect(Collectors.toList()), true);
		}
		if (CollUtil.isEmpty(stbzList)) {
			return;
		}
		Map<String, GoodsInfo> goodsInfoMap = stbzList.stream().collect(Collectors.toMap(GoodsInfo::getThirdId, Function.identity()));
		List<String> collect = stbzList.stream().map(GoodsInfo::getThirdId).collect(Collectors.toList());
		JSONObject details = supplyChainService.detail(collect);
		JSONArray data = details.getJSONArray("data");
		for (Object datum : data) {
			JSONObject json = (JSONObject) datum;
			String id = json.getString("id");
			GoodsInfo goodsInfo = goodsInfoMap.get(id);
			if (goodsInfo != null ) {
				goodsInfo.setLastSyncTime(new Date());
				goodsInfo.setGoodsCover(json.getString("cover"));
				goodsInfo.setGoodsName(json.getString("title"));
//				goodsInfo.setIsSale(json.getIntValue("status"));
				goodsInfo.setTotalSales(json.getIntValue("sale"));
				goodsInfo.setTotalStock(json.getInteger("total_stock"));
				goodsInfo.setUnit(json.getString("unit"));
				goodsInfo.setOrigin(json.getString("origin"));
				goodsInfo.setActivityRate(BigDecimal.valueOf(json.getDoubleValue("activity_rate")));
				goodsInfo.setMarketPrice(BigDecimal.valueOf(json.getDoubleValue("market_price")).divide(new BigDecimal(100)));
				goodsInfo.setActivityPrice(BigDecimal.valueOf(json.getDoubleValue("activity_price")).divide(new BigDecimal(100)));
				BigDecimal agreementPrice = BigDecimal.valueOf(json.getDoubleValue("agreement_price")).divide(new BigDecimal(100));
				// 计算协议价
				goodsInfo.setAgreementPrice(supplyChainService.calAgreementPrice(agreementPrice));
//				goodsInfo.setCostPrice(BigDecimal.valueOf(json.getDoubleValue("cost_price")).divide(new BigDecimal(100)));
				if (isUpdateSalePrice) {
					// 创客专区的商品不更新价格
					if (!ZoneTypeEnum.GUEST.getValue().equals(goodsInfo.getZoneType())) {
						// 计算售价
						BigDecimal salePrice = this.calSalePrice(goodsInfo.getPromotionRate(), goodsInfo.getAgreementPrice());
						goodsInfo.setSalePrice(salePrice);
					}
					goodsInfo.setGuidePrice(BigDecimal.valueOf(json.getDoubleValue("guide_price")).divide(new BigDecimal(100)));
				}
				this.updateById(goodsInfo);

				// 更新规格信息
				List<GoodsSpec> specList = goodsSpecService.lambdaQuery().eq(GoodsSpec::getGoodsId, goodsInfo.getId()).list();
				specList = removeDuplication(specList);
				if (CollUtil.isNotEmpty(specList)) {
					Map<String, GoodsSpec> specMap = specList.stream().collect(Collectors.toMap(GoodsSpec::getOptionId, Function.identity()));
					JSONArray jsonArray = json.getJSONObject("specs").getJSONArray("options");
					List<GoodsSpec> addList = new ArrayList<>();
					for (int i = 0; i < jsonArray.size(); i++) {
						JSONObject specJson = jsonArray.getJSONObject(i);
						String optionId = specJson.getString("id");
						GoodsSpec goodsSpec = specMap.get(optionId);
						// 已存在的规格执行更新，不存在则新增
						if (goodsSpec != null) {
							this.specConvert(goodsSpec, specJson, goodsInfo.getZoneType(), isUpdateSalePrice);
							goodsSpecService.updateById(goodsSpec);
							// 将已存在的从集合中移除，未移除的需要删除
							specMap.remove(optionId);
						} else {
							goodsSpec = new GoodsSpec();
							goodsSpec.setPromotionRate(goodsInfo.getPromotionRate());
							this.specConvert(goodsSpec, specJson, goodsInfo.getZoneType(), true);
							goodsSpec.setGoodsId(goodsInfo.getId());
							goodsSpec.setGoodsNo(goodsInfo.getGoodsNo());
							goodsSpec.setIsDefault(1);
							goodsSpec.setOptionId(optionId);
							goodsSpec.setNeedIntegral(goodsSpec.getSalePrice());
							goodsSpec.setObtainIntegral(goodsSpec.getSalePrice());
							goodsSpec.setIsSale(1);
							goodsSpec.setIsPackage(0);
							addList.add(goodsSpec);
						}
					}
					if (CollUtil.isNotEmpty(specMap)) {
						specMap.forEach((k,v) -> goodsSpecService.removeById(v.getId()));
					}
					if (CollUtil.isNotEmpty(addList)) {
						// 去除重复数据，防止第三方返回的数据有重复的
						addList = removeDuplication(addList);
						addList.forEach(e -> goodsSpecService.submit(e));
					}
				}
			}
		}
	}

	/**
	 * 去除重复数据
	 * @param specList
	 */
	private List<GoodsSpec> removeDuplication(List<GoodsSpec> specList) {
		if (CollUtil.isEmpty(specList)) {
			return specList;
		}
		Map<String, GoodsSpec> map = new HashMap<>();
		for (GoodsSpec goodsSpec : specList) {
			GoodsSpec spec = map.get(goodsSpec.getOptionId());
			if (spec == null) {
				map.put(goodsSpec.getOptionId(), goodsSpec);
			} else {
				if (spec.getId() != null) {
					goodsSpecService.removeById(goodsSpec.getId());
				}
			}
		}
		return new ArrayList<>(map.values());
	}

	private void specConvert(GoodsSpec goodsSpec, JSONObject specJson, Integer zoneType, boolean isUpdateSalePrice) {
		goodsSpec.setCover(specJson.getString("image"));
		goodsSpec.setSpceName(specJson.getString("spec_value_names"));
		goodsSpec.setIsSale(specJson.getIntValue("status"));
		goodsSpec.setStock(specJson.getIntValue("stock"));
		goodsSpec.setActivityRate(BigDecimal.valueOf(specJson.getDoubleValue("activity_rate")));
		goodsSpec.setMarketPrice(BigDecimal.valueOf(specJson.getDoubleValue("market_price")).divide(new BigDecimal(100)));
		goodsSpec.setActivityPrice(BigDecimal.valueOf(specJson.getDoubleValue("activity_price")).divide(new BigDecimal(100)));
		BigDecimal agreementPriceSpec = BigDecimal.valueOf(specJson.getDoubleValue("agreement_price")).divide(new BigDecimal(100));
		goodsSpec.setAgreementPrice(supplyChainService.calAgreementPrice(agreementPriceSpec));
		if (isUpdateSalePrice) {
			goodsSpec.setCostPrice(BigDecimal.valueOf(specJson.getDoubleValue("cost_price")).divide(new BigDecimal(100)));
			BigDecimal specSalePrice = this.calSalePrice(goodsSpec.getPromotionRate(), goodsSpec.getAgreementPrice());
			goodsSpec.setSalePrice(specSalePrice);
			goodsSpec.setGuidePrice(BigDecimal.valueOf(specJson.getDoubleValue("guide_price")).divide(new BigDecimal(100)));
		}
		if (ZoneTypeEnum.GUEST.getValue().equals(zoneType)) {
			goodsSpec.setSalePrice(new BigDecimal("99.00"));
		}
	}

	@Override
	public void syncSpecPromotion(Integer start, Integer end) {
		List<GoodsSpec> updateList = new ArrayList<>();
		List<Long> ids = this.getBaseMapper().mySelect(start, end);
		log.info(String.valueOf(ids.size()));
		List<List<Long>> partition = ListUtil.partition(ids, 1000);
		for (List<Long> longs : partition) {
			List<GoodsInfo> list = this.lambdaQuery().in(GoodsInfo::getId, longs).list();
			List<GoodsSpec> goodsSpecs = goodsSpecService.lambdaQuery().in(GoodsSpec::getGoodsId, longs).list();
			Map<Long, List<GoodsSpec>> map = goodsSpecs.stream().collect(Collectors.groupingBy(GoodsSpec::getGoodsId));
			for (GoodsInfo goodsInfo : list) {
				List<GoodsSpec> specs = map.get(goodsInfo.getId());
				for (GoodsSpec spec : specs) {
					BigDecimal agreementPrice = supplyChainService.calAgreementPrice(spec.getAgreementPrice());
					BigDecimal specSalePrice = this.calSalePrice(goodsInfo.getPromotionRate(), agreementPrice);
					GoodsSpec updateRecord = new GoodsSpec();
					updateRecord.setId(spec.getId());
					updateRecord.setAgreementPrice(agreementPrice);
					updateRecord.setPromotionRate(goodsInfo.getPromotionRate());
					updateRecord.setSalePrice(specSalePrice);
					updateRecord.setGuidePrice(specSalePrice);
					updateList.add(updateRecord);
				}
			}
		}
		log.info("updateList -> " + updateList.size());
		List<List<GoodsSpec>> partition1 = ListUtil.partition(updateList, 1000);
		int index = 1;
		log.info(String.valueOf(partition1.size()));
		for (List<GoodsSpec> goodsSpecs : partition1) {
			log.info("update " + index);
			index++;
			goodsSpecService.updateBatchById(goodsSpecs);
		}
	}

	/**
	 * 使用redis生成商品编号
	 * @return
	 */
	private String generateGoodsNo() {
		RedisAtomicLong redisAtomicLong = new RedisAtomicLong(GOODS_NO_PREFIX, redisTemplate.getConnectionFactory());
		long seq = redisAtomicLong.incrementAndGet();
		return "1" + String.format("%07d", seq);
	}

}
