/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.modules.redenvelopemall.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springblade.common.cache.ParamCache;
import org.springblade.common.constant.CommonConstant;
import org.springblade.common.enums.OrderTypeEnum;
import org.springblade.common.enums.SettleTypeEnum;
import org.springblade.common.enums.TicketTypeEnum;
import org.springblade.core.log.exception.ServiceException;
import org.springblade.core.log.logger.BladeLogger;
import org.springblade.modules.finance.entity.Order;
import org.springblade.modules.finance.entity.OrderGoods;
import org.springblade.modules.finance.mapper.OrderGoodsMapper;
import org.springblade.modules.finance.mapper.OrderMapper;
import org.springblade.modules.platform.dto.AddUserTicketDTO;
import org.springblade.modules.platform.service.IUserTicketService;
import org.springblade.modules.redenvelopemall.entity.GoodsInfo;
import org.springblade.modules.redenvelopemall.entity.RedAccelerateRecord;
import org.springblade.modules.redenvelopemall.entity.RedIntegral;
import org.springblade.modules.redenvelopemall.mapper.GoodsInfoMapper;
import org.springblade.modules.redenvelopemall.mapper.RedAccelerateRecordMapper;
import org.springblade.modules.redenvelopemall.service.IGoodsInfoService;
import org.springblade.modules.redenvelopemall.service.IRedAccelerateRecordService;
import org.springblade.modules.redenvelopemall.vo.RedIntegralVO;
import org.springblade.modules.redenvelopemall.mapper.RedIntegralMapper;
import org.springblade.modules.redenvelopemall.service.IRedIntegralService;
import org.springblade.core.mp.base.BaseServiceImpl;
import org.springblade.modules.shopping.entity.Goods;
import org.springblade.modules.shopping.mapper.GoodsMapper;
import org.springblade.modules.system.entity.UserApp;
import org.springblade.modules.system.mapper.UserAppMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.beans.Transient;
import java.math.BigDecimal;

/**
 * 红色积分变更记录表 服务实现类
 *
 * @author BladeX
 * @since 2022-05-25
 */
@Service
public class RedIntegralServiceImpl extends ServiceImpl<RedIntegralMapper, RedIntegral> implements IRedIntegralService {

	@Resource
	private RedAccelerateRecordMapper recordMapper;//加速积分记录mapper

	@Resource
	private UserAppMapper userAppMapper;//用户积分记录表

	@Resource
	private IUserTicketService ticketService;

	@Resource
	private OrderGoodsMapper orderGoodsMapper;//订单商品mapper

	@Resource
	private OrderMapper orderMapper;

	@Resource
	private IGoodsInfoService goodsService;

	@Resource
	private BladeLogger logger;

	@Override
	public IPage<RedIntegralVO> selectRedIntegralPage(IPage<RedIntegralVO> page, RedIntegralVO redIntegral) {
		return page.setRecords(baseMapper.selectRedIntegralPage(page, redIntegral));
	}

	@Override
	public BigDecimal countYesterdayAccelerate(){
		return recordMapper.countYesterdayAccelerate();
	}

	@Transient
	@Override
	public void calculateIntegral(String outTradeNo) {
		//通过订单号获取订单商品信息
		OrderGoods orderGoods = orderGoodsMapper.selectOne(Wrappers.lambdaQuery(OrderGoods.class).eq(OrderGoods::getOutTradeNo, outTradeNo));
		//获取商品信息
		GoodsInfo goodsInfo = goodsService.get(orderGoods.getGoodsId());
		//通过订单号获取订单信息
		Order order = orderMapper.selectOne(Wrappers.lambdaQuery(Order.class).eq(Order::getOutTradeNo, outTradeNo));
		if(ObjectUtil.isNull(orderGoods) || ObjectUtil.isNull(order)){
			logger.info("红色积分计算失败","未查询到订单信息，商品订单id：" + outTradeNo);
			throw new ServiceException("未查询到订单信息，红色积分计算失败！");
		}
		//通过订单获取用户信息，获得用户的上级用户
		UserApp userApp = userAppMapper.selectOne(Wrappers.lambdaQuery(UserApp.class).eq(UserApp::getUserId, order.getUserId()));
		if(ObjectUtil.isNull(userApp)){
			throw new ServiceException("未查询到该用户，无需计算");
		}
		//通过订单商品红色积分新增用户红色积分
		Double integral = userApp.getIntegral();//用户原有红色积分
		//计算新增红色积分
		BigDecimal nowIntegral = new BigDecimal(integral).add(new BigDecimal(order.getTotalTicket())).setScale(2, BigDecimal.ROUND_DOWN);
		//更新用户信息表与记录新增积分
		RedIntegral redIntegral = new RedIntegral();
		redIntegral.setBeforeIntegral(new BigDecimal(integral));
		redIntegral.setLastIntegral(nowIntegral);
		redIntegral.setRatio(new BigDecimal(0));
		redIntegral.setDataId(order.getId());
		redIntegral.setUserId(order.getUserId());
		redIntegral.setChangeIntegral(new BigDecimal(order.getTotalTicket()));
		redIntegral.setType(1);
		redIntegral.setSrc(1);
		redIntegral.setRemark("订单新增红色积分");
		this.baseMapper.insert(redIntegral);
		userApp.setIntegral(nowIntegral.doubleValue());
		//创客订单
		if(OrderTypeEnum.MAKE.getValue().equals(order.getOrderType())){
			if(ObjectUtil.isNotEmpty(userApp.getIsRole()) && userApp.getIsRole().equals(1)){
				userApp.setIsRole(3);
			}else{
				userApp.setIsRole(2);
			}
		}
		//城市合伙人订单
		if(OrderTypeEnum.PARTNER.getValue().equals(order.getOrderType())){
			if(ObjectUtil.isNotEmpty(userApp.getIsRole()) && userApp.getIsRole().equals(2)){
				userApp.setIsRole(3);
			}else{
				userApp.setIsRole(1);
			}
		}
		userAppMapper.updateById(userApp);
		//通过订单商品金额计算上级用户加速积分
		BigDecimal oneRadio = new BigDecimal(ParamCache.getValue(CommonConstant.INTEGRAL_ACCELERATE_RATIO));//一级加速比例
		//判断是否为特殊商品且加速红包比例大于0则使用加速红包比例
		if(ObjectUtil.isNotEmpty(goodsInfo.getIsSpecial()) && goodsInfo.getIsSpecial() == 1 && ObjectUtil.isNotEmpty(goodsInfo.getSpeedRatio()) && goodsInfo.getSpeedRatio().compareTo(new BigDecimal(0)) == 1){
			oneRadio = goodsInfo.getSpeedRatio();
		}
		//获取上级用户用于计算是否能可以进行加速
		UserApp puserApp = userAppMapper.selectOne(Wrappers.lambdaQuery(UserApp.class).eq(UserApp::getUserId, userApp.getParentId()));
		//判断必须为上级用户拥有红色积分才能生成记录,城市合伙人订单不能获得加速积分,且等于1580订单时正常订单不产生加速
		if(puserApp.getIntegral() > 0 && !OrderTypeEnum.PARTNER.getValue().equals(order.getOrderType()) && !OrderTypeEnum.LINKED.getValue().equals(order.getOrderType())){
			//计算加速积分
			BigDecimal addIntegral = new BigDecimal(order.getTotalFee()).multiply(oneRadio).setScale(2, BigDecimal.ROUND_DOWN);
			RedAccelerateRecord redAccelerateRecord = new RedAccelerateRecord();
			redAccelerateRecord.setAmount(addIntegral);
			redAccelerateRecord.setAccelerateUserId(order.getUserId());
			redAccelerateRecord.setBenefitUserId(userApp.getParentId());
			redAccelerateRecord.setDataId(order.getId());
			//判断加速积分是否大于用户拥有积分
			if(new BigDecimal(puserApp.getIntegral()).compareTo(addIntegral) == -1){
				//如果用户积分小于加速积分，那么默认加速积分等于用户当前积分
				addIntegral = new BigDecimal(puserApp.getIntegral());
				redAccelerateRecord.setRemark("加速积分大于用户积分，所以当前加速积分为用户积分：" + addIntegral);
			}
			redAccelerateRecord.setAccelerateIntegral(addIntegral);
			redAccelerateRecord.setRatio(oneRadio);
			redAccelerateRecord.setType(0);//此处为一级加速
			recordMapper.insert(redAccelerateRecord);
		}
		//当为城市合伙人时推荐人获得10%的绿色积分
		if(OrderTypeEnum.PARTNER.getValue().equals(order.getOrderType())){
			BigDecimal addScore = orderGoods.getSalePrice().multiply(new BigDecimal(0.1)).setScale(2, BigDecimal.ROUND_DOWN);
			ticketService.addScoreBySrc(puserApp.getUserId(), addScore, 6, "直推人员购买城市合伙人积分赠送", TicketTypeEnum.INCOME);
		}


		//todo 判断用户的上两级是否为城市合伙人，后期加入时更改，如果当前上级为城市合伙人，那么就不需要两级了分润了


	}

	@Override
	public void subIntegral(Long userId, BigDecimal subIntegral, Integer src, Long dataId) {
		UserApp userApp = userAppMapper.selectOne(Wrappers.lambdaQuery(UserApp.class).eq(UserApp::getUserId, userId));
		//通过订单商品红色积分新增用户红色积分
		Double integral = userApp.getIntegral();//用户原有红色积分
		//计算减红色积分
		BigDecimal nowIntegral = new BigDecimal(integral).subtract(subIntegral).setScale(2, BigDecimal.ROUND_DOWN);
		//更新用户信息表与记录新增积分
		RedIntegral redIntegral = new RedIntegral();
		redIntegral.setBeforeIntegral(new BigDecimal(integral));
		redIntegral.setLastIntegral(nowIntegral);
		redIntegral.setRatio(new BigDecimal(0));
		redIntegral.setDataId(dataId);
		redIntegral.setUserId(userId);
		redIntegral.setSrc(src);
		redIntegral.setChangeIntegral(subIntegral);
		redIntegral.setType(2);
		if(src == 2){
			redIntegral.setRemark("随机红包扣除积分");
		}else{
			redIntegral.setRemark("专属红包扣除积分");
		}
		this.baseMapper.insert(redIntegral);
		userApp.setIntegral(nowIntegral.doubleValue());
		userAppMapper.updateById(userApp);
	}

	@Override
	public BigDecimal getExclusive(Long userId) {
		return recordMapper.getExclusive(userId);
	}

	@Override
	public BigDecimal getAllExclusive(Long userId) {
		return recordMapper.getExclusive(userId);
	}

	@Override
	public void addUserScore(AddUserTicketDTO addUserTicketDTO) {
		//通过订单获取用户信息，获得用户的上级用户
		UserApp userApp = userAppMapper.selectOne(Wrappers.lambdaQuery(UserApp.class).eq(UserApp::getUserId, addUserTicketDTO.getUserId()));
		if(ObjectUtil.isNull(userApp)){
			throw new ServiceException("未查询到该用户");
		}
		//通过订单商品红色积分新增用户红色积分
		Double integral = userApp.getIntegral();//用户原有红色积分
		//计算新增红色积分
		BigDecimal nowIntegral = new BigDecimal(integral).add(BigDecimal.valueOf(addUserTicketDTO.getScore())).setScale(2, BigDecimal.ROUND_DOWN);
		//更新用户信息表与记录新增积分
		RedIntegral redIntegral = new RedIntegral();
		redIntegral.setBeforeIntegral(new BigDecimal(integral));
		redIntegral.setLastIntegral(nowIntegral);
		redIntegral.setRatio(new BigDecimal(0));
		redIntegral.setUserId(addUserTicketDTO.getUserId());
		redIntegral.setChangeIntegral(BigDecimal.valueOf(addUserTicketDTO.getScore()));
		redIntegral.setType(1);
		redIntegral.setSrc(6);
		redIntegral.setRemark(addUserTicketDTO.getRemark());
		this.baseMapper.insert(redIntegral);
		userApp.setIntegral(nowIntegral.doubleValue());
		userAppMapper.updateById(userApp);
	}

	@Transactional(rollbackFor = Exception.class)
	@Override
	public void orderReturn(String outTradeNo) {
		Order order = orderMapper.selectOne(Wrappers.lambdaQuery(Order.class).eq(Order::getOutTradeNo, outTradeNo));
		if (order == null) {
			throw new ServiceException("订单不存在");
		}
		//通过订单获取用户信息，获得用户的上级用户
		UserApp userApp = userAppMapper.selectOne(Wrappers.lambdaQuery(UserApp.class).eq(UserApp::getUserId, order.getUserId()));
		if(ObjectUtil.isNull(userApp)){
			throw new ServiceException("未查询到该用户，无需计算");
		}
		//通过订单商品红色积分新增用户红色积分
		Double integral = userApp.getIntegral();//用户原有红色积分
		//计算新增红色积分
		BigDecimal nowIntegral = new BigDecimal(integral).add(new BigDecimal(order.getTotalTicket())).setScale(2, BigDecimal.ROUND_DOWN);
		//更新用户信息表与记录新增积分
		RedIntegral redIntegral = new RedIntegral();
		redIntegral.setBeforeIntegral(new BigDecimal(integral));
		redIntegral.setLastIntegral(nowIntegral);
		redIntegral.setRatio(new BigDecimal(0));
		redIntegral.setDataId(order.getId());
		redIntegral.setUserId(order.getUserId());
		redIntegral.setChangeIntegral(new BigDecimal(order.getTotalTicket()));
		redIntegral.setType(1);
		redIntegral.setSrc(1);
		redIntegral.setRemark("订单退还");
		this.baseMapper.insert(redIntegral);
		userApp.setIntegral(nowIntegral.doubleValue());
		userAppMapper.updateById(userApp);
	}

	@Override
	public void refundDeduct(Order order) {
		UserApp userApp = userAppMapper.selectOne(Wrappers.lambdaQuery(UserApp.class).eq(UserApp::getUserId, order.getUserId()));
		//通过订单商品红色积分新增用户红色积分
		Double integral = userApp.getIntegral();//用户原有红色积分
		//计算减红色积分

		BigDecimal nowIntegral = new BigDecimal(integral).subtract(BigDecimal.valueOf(order.getTotalTicket())).setScale(2, BigDecimal.ROUND_DOWN);
		//更新用户信息表与记录积分明细
		RedIntegral redIntegral = new RedIntegral();
		redIntegral.setBeforeIntegral(new BigDecimal(integral));
		redIntegral.setLastIntegral(nowIntegral);
		redIntegral.setRatio(new BigDecimal(0));
		redIntegral.setDataId(order.getId());
		redIntegral.setUserId(userApp.getUserId());
		redIntegral.setSrc(5);
		redIntegral.setChangeIntegral(BigDecimal.valueOf(order.getTotalTicket()));
		redIntegral.setType(2);
		redIntegral.setRemark("订单退款扣除");
		this.baseMapper.insert(redIntegral);
		userApp.setIntegral(nowIntegral.doubleValue());
		userAppMapper.updateById(userApp);
	}
}
