package org.springblade.modules.system.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springblade.common.enums.RedMoneySourceEnum;
import org.springblade.common.param.ComResult;
import org.springblade.common.utils.RedisTool;
import org.springblade.core.log.logger.BladeLogger;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.modules.platform.service.UserAppService;
import org.springblade.modules.system.entity.RedMoneyRecord;
import org.springblade.modules.system.entity.UserApp;
import org.springblade.modules.system.mapper.RedMoneyRecordMapper;
import org.springblade.modules.system.service.INowMoneyRecordService;
import org.springblade.modules.system.service.IRedMoneyRecordService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Date;
import java.util.Objects;


@Service
public class IRedMoneyRecordServiceImpl extends ServiceImpl<RedMoneyRecordMapper, RedMoneyRecord> implements IRedMoneyRecordService {

	private static final Logger log = LoggerFactory.getLogger(INowMoneyRecordService.class);

	@Resource
	private UserAppService userAppService;

	@Resource
	private BladeLogger logger;

	@Resource
	private RedisTool redisTool;


	/**
	 * 创建红包金额变动记录及变更红包余额
	 *
	 * @param userId          需要变更红包余额的用户id
	 * @param sourceEnum      红包余额变动类型
	 * @param fluctuantAmount 变动金额
	 * @param chargeAmount    手续费
	 * @param dataId          数据id
	 * @return 结果
	 */
	@Override
	@Transactional
	public ComResult redMoneyUpdateRecord(Long userId,
										  RedMoneySourceEnum sourceEnum,
										  BigDecimal fluctuantAmount,
										  BigDecimal chargeAmount,
										  Long dataId) {

		ComResult comResult = new ComResult();
		if (RedMoneySourceEnum.WITHDRAWAL_SUCCEED.equals(sourceEnum)){
			comResult.setFlag(false);
			comResult.setMsg("操作无效!");
			return comResult;
		}

		//查询用户钱包
		LambdaQueryWrapper<UserApp> query = new LambdaQueryWrapper<>();
		query.eq(UserApp::getUserId, userId);
		query.eq(UserApp::getState, 1);
		UserApp userApp = userAppService.getOne(query);
		String str = Objects.isNull(dataId) ? "" : "来源ID：" + dataId;
		if (Objects.isNull(userApp)) {
			comResult.setFlag(false);
			comResult.setMsg("用户钱包不存在或被冻结!");
			log.info(sourceEnum.getName() + "失败", "用户钱包不存在或被冻结！" + str);
			logger.info(sourceEnum + "失败", "用户钱包不存在或被冻结！" + str);
			return comResult;
		}
		//验证红包余额和最新的记录数据是否异常
		RedMoneyRecord redMoneyRecord = this.getLastRecordByUserId(userId);
		if (Objects.nonNull(redMoneyRecord)) {
			if (!userApp.getRedMoney().equals(redMoneyRecord.getAfterMoney())) {
				comResult.setFlag(false);
				comResult.setMsg("红包数据异常!");
				log.info(sourceEnum.getName() + "失败", "红包数据异常！" + str);
				logger.info(sourceEnum + "失败", "红包数据异常！" + str);
				return comResult;
			}
		}
		try {
			if (!redisTool.getRedisLock("redMoney:"+ userApp.getId(),"redMoney")) {
				throw new RuntimeException("系统繁忙，请稍后再试！");
			}
			//计算变化后的红包余额
			BigDecimal beforeMoney = new BigDecimal(0);
			BigDecimal afterMoney = new BigDecimal(0);
			if(Objects.nonNull(userApp.getRedMoney())){
				beforeMoney = userApp.getRedMoney();
			}
			//判断当前记录为  收入
			if (sourceEnum.getType() == 1) {
				//计算变动后的金额
				afterMoney = beforeMoney.add(fluctuantAmount);

			}
			//判断当前记录为  支出
			if (sourceEnum.getType() == 2) {
				//计算变动后的金额
				afterMoney = beforeMoney.subtract(fluctuantAmount);
				//判断计算后的金额是否小于0
				if (afterMoney.compareTo(new BigDecimal(0)) < 0) { //余额不足
					comResult.setFlag(false);
					comResult.setMsg("用户钱包余额不足!");
					log.info(sourceEnum.getName() + "失败", "用户红包余额不足！钱包id：" + userApp.getId() + str);
					logger.info(sourceEnum + "失败", "用户红包余额不足！钱包id：" + str);
					return comResult;
				}
			}
			//新增红包记录
			RedMoneyRecord newRedMoneyRecord = new RedMoneyRecord();
			newRedMoneyRecord.setBeforeMoney(beforeMoney);
			newRedMoneyRecord.setAfterMoney(afterMoney);
			newRedMoneyRecord.setType(sourceEnum.getType());
			newRedMoneyRecord.setFluctuantAmount(fluctuantAmount);
			newRedMoneyRecord.setChargeAmount(chargeAmount);
			newRedMoneyRecord.setTaxAmount(new BigDecimal(0));
			newRedMoneyRecord.setSource(sourceEnum.getName());
			newRedMoneyRecord.setSourceCode(sourceEnum.getValue());
			newRedMoneyRecord.setUserId(userId);
			newRedMoneyRecord.setUserAppId(userApp.getId());
			newRedMoneyRecord.setDataId(dataId);
			newRedMoneyRecord.setCreateUserId(AuthUtil.getUserId());
			newRedMoneyRecord.setUpdateUserId(AuthUtil.getUserId());
			Date date = new Date();
			newRedMoneyRecord.setCreateTime(date);
			newRedMoneyRecord.setUpdateTime(date);
			newRedMoneyRecord.setIsDelete(0);
			newRedMoneyRecord.setParamStr("RecordParam{" +
				"sourceEnum=" + sourceEnum +
				", fluctuantAmount=" + fluctuantAmount +
				", chargeAmount=" + chargeAmount +
				", userId=" + userId +
				", dataId=" + dataId +
				'}');
			//修改红包余额
			userApp.setRedMoney(afterMoney);
			this.save(newRedMoneyRecord);
			userAppService.saveOrUpdate(userApp);
			//返回结果
			comResult.setFlag(true);
			comResult.setMsg("操作成功！");
			return comResult;
		}catch (Exception e){
			comResult.setFlag(false);
			comResult.setMsg(e.getMessage());
			log.info(sourceEnum.getName() + "失败", "e.getMessage()" + userApp.getId() + str);
			logger.info(sourceEnum + "失败", "e.getMessage()" + str);
			return comResult;
		}finally {
			redisTool.unRedisLock("redMoney:"+ userApp.getId());
		}
	}

	@Override
	public ComResult redMoneySuccRecord(Long userId, Long dataId) {
		ComResult comResult = new ComResult();

		RedMoneyRecord redMoneyRecord = getRecordByUserIdAndDataId(userId,dataId);
		if (Objects.isNull(redMoneyRecord)){
			comResult.setFlag(false);
			comResult.setMsg("没有查询到相应的提现记录！");
			return comResult;
		}
		redMoneyRecord.setSource(RedMoneySourceEnum.WITHDRAWAL_SUCCEED.getName());
		redMoneyRecord.setSourceCode(RedMoneySourceEnum.WITHDRAWAL_SUCCEED.getValue());
		redMoneyRecord.setUpdateTime(new Date());
		updateById(redMoneyRecord);

//		UserApp userApp = userAppService.getByUserId(userId);
//		RedMoneyRecord newRedMoneyRecord = new RedMoneyRecord();
//		newRedMoneyRecord.setBeforeMoney(userApp.getRedMoney());
//		newRedMoneyRecord.setAfterMoney(userApp.getRedMoney());
//		newRedMoneyRecord.setType(RedMoneySourceEnum.WITHDRAWAL_SUCCEED.getType());
//		newRedMoneyRecord.setFluctuantAmount(redMoneyRecord.getFluctuantAmount());
//		newRedMoneyRecord.setChargeAmount(redMoneyRecord.getChargeAmount());
//		newRedMoneyRecord.setTaxAmount(redMoneyRecord.getTaxAmount());
//		newRedMoneyRecord.setSource(RedMoneySourceEnum.WITHDRAWAL_SUCCEED.getName());
//		newRedMoneyRecord.setSourceCode(RedMoneySourceEnum.WITHDRAWAL_SUCCEED.getValue());
//		newRedMoneyRecord.setUserId(userId);
//		newRedMoneyRecord.setUserAppId(userApp.getId());
//		newRedMoneyRecord.setParamStr("userId="+userId +",dataId="+dataId);
//		newRedMoneyRecord.setDataId(dataId);
//		newRedMoneyRecord.setCreateUserId(AuthUtil.getUserId());
//		newRedMoneyRecord.setUpdateUserId(AuthUtil.getUserId());
//		Date date = new Date();
//		newRedMoneyRecord.setCreateTime(date);
//		newRedMoneyRecord.setIsDelete(0);
//		this.save(newRedMoneyRecord);
		comResult.setFlag(true);
		comResult.setMsg("操作成功！");
		return comResult;
	}

	private RedMoneyRecord getRecordByUserIdAndDataId(Long userId, Long dataId) {
		LambdaQueryWrapper<RedMoneyRecord> query = new LambdaQueryWrapper<>();
		query.eq(RedMoneyRecord::getIsDelete, 0);
		query.eq(RedMoneyRecord::getUserId, userId);
		query.eq(RedMoneyRecord::getDataId, dataId);
		query.eq(RedMoneyRecord::getSourceCode,RedMoneySourceEnum.WITHDRAWAL.getValue());
		return this.getOne(query);
	}


	private RedMoneyRecord getLastRecordByUserId(Long userId) {
		LambdaQueryWrapper<RedMoneyRecord> query = new LambdaQueryWrapper<>();
		query.eq(RedMoneyRecord::getIsDelete, 0);
		query.eq(RedMoneyRecord::getUserId, userId);
		query.orderByDesc(RedMoneyRecord::getCreateTime);
		query.last("limit 1");
		return this.getOne(query);
	}
}
