/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.modules.system.service.impl;

import cn.hutool.core.util.ObjectUtil;
import org.springblade.common.cache.ParamCache;
import org.springblade.common.constant.CommonConstant;
import org.springblade.common.enums.RedMoneySourceEnum;
import org.springblade.common.enums.WithdrawalStatusEnum;
import org.springblade.common.param.ComResult;
import org.springblade.common.utils.RedisTool;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.modules.api.service.IZhzxAliService;
import org.springblade.modules.platform.service.UserAppService;
import org.springblade.modules.system.entity.MallWithdrawalRecord;
import org.springblade.modules.system.entity.PayLog;
import org.springblade.modules.system.entity.User;
import org.springblade.modules.system.entity.UserApp;
import org.springblade.modules.system.service.IRedMoneyRecordService;
import org.springblade.modules.system.service.IUserService;
import org.springblade.modules.system.vo.MallWithdrawalRecordVO;
import org.springblade.modules.system.mapper.MallWithdrawalRecordMapper;
import org.springblade.modules.system.service.IMallWithdrawalRecordService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Date;

/**
 *  服务实现类
 *
 * @author BladeX
 * @since 2022-11-24
 */
@Service
public class MallWithdrawalRecordServiceImpl extends ServiceImpl<MallWithdrawalRecordMapper, MallWithdrawalRecord> implements IMallWithdrawalRecordService {

	@Resource
	private IRedMoneyRecordService redMoneyRecordService;

	@Resource
	private IUserService userService;

	@Resource
	private RedisTool redisTool;

	@Resource
	private UserAppService userAppService;

	@Resource
	private IZhzxAliService aliService;

	@Override
	public IPage<MallWithdrawalRecordVO> selectMallWithdrawalRecordPage(IPage<MallWithdrawalRecordVO> page, MallWithdrawalRecordVO mallWithdrawalRecord) {
		return page.setRecords(baseMapper.selectMallWithdrawalRecordPage(page, mallWithdrawalRecord));
	}

	@Override
	@Transactional
	public void createMallWithdrawalRecord(BigDecimal amount) {
		MallWithdrawalRecord mallWithdrawalRecord = new MallWithdrawalRecord();
		mallWithdrawalRecord.setWithdrawalAmount(amount);
		mallWithdrawalRecord.setStatus(WithdrawalStatusEnum.TO_AUDIT.getValue());
		mallWithdrawalRecord.setCreateUserId(AuthUtil.getUserId());
		//红包提现手续费
		BigDecimal redWithdrawalRatio = new BigDecimal(ParamCache.getValue(CommonConstant.RED_WITHDRAWAL_RATIO));
		mallWithdrawalRecord.setChargeAmount(amount.multiply(redWithdrawalRatio));
		User user = userService.getById(AuthUtil.getUserId());
		mallWithdrawalRecord.setCreateUserName(user.getRealName());
		baseMapper.insert(mallWithdrawalRecord);
		//红包记录保存
		redMoneyRecordService.redMoneyUpdateRecord(AuthUtil.getUserId(), RedMoneySourceEnum.WITHDRAWAL, amount, mallWithdrawalRecord.getChargeAmount(), mallWithdrawalRecord.getId());
	}

	@Override
	public void confirmAudit(Long id) {
		MallWithdrawalRecord record = baseMapper.selectById(id);
		record.setStatus(WithdrawalStatusEnum.PASS.getValue());
		record.setAuditMsg("同意申请");
		record.setAuditTime(new Date());
		record.setAuditUserId(AuthUtil.getUserId());
		User user = userService.getById(AuthUtil.getUserId());
		record.setAuditUserName(user.getRealName());
		//生成提现成功记录
		redMoneyRecordService.redMoneySuccRecord(record.getCreateUserId(), record.getId());
		//更新提现记录
		baseMapper.updateById(record);
		//todo 提现进行打款操作
	}

	@Override
	@Transactional
	public void updateMallWithdrawalRecord(Long id, WithdrawalStatusEnum withdrawalStatusEnum,String auditMsg) {
		try {
			if (!redisTool.getRedisLock("mallWithdrawalRecord:"+ id,"mallWithdrawalRecord")) {
				throw new RuntimeException("系统繁忙，请稍后再试！");
			}
			MallWithdrawalRecord record = baseMapper.selectById(id);
			if(ObjectUtil.isNull(record) || record.getStatus() != 0){
				throw new RuntimeException("记录异常，请稍后再试");
			}
			record.setStatus(withdrawalStatusEnum.getValue());
			record.setAuditMsg(auditMsg);
			record.setAuditTime(new Date());
			record.setAuditUserId(AuthUtil.getUserId());
			User user = userService.getById(AuthUtil.getUserId());
			record.setAuditUserName(user.getRealName());
			ComResult comResult = null;
			//生成红包余额记录
			if(withdrawalStatusEnum.equals(WithdrawalStatusEnum.PASS)){//申请通过
				redMoneyRecordService.redMoneySuccRecord(record.getCreateUserId(), record.getId());
				UserApp userApp = userAppService.getByUserId(record.getCreateUserId());
				User nowUser = userService.getById(record.getCreateUserId());
				PayLog pay = aliService.pay(2, record.getCreateUserId(), userApp.getAlipayUserId(), nowUser.getRealName(), record.getWithdrawalAmount().subtract(record.getChargeAmount()), "红包提现", "红包提现");
				if(pay.getIsSuccess() != 1){
					throw new RuntimeException(pay.getFailureReason());
				}
			}else if (withdrawalStatusEnum.equals(WithdrawalStatusEnum.REJECT)){//驳回申请
				comResult = redMoneyRecordService.redMoneyUpdateRecord(AuthUtil.getUserId(), RedMoneySourceEnum.WITHDRAWAL_FAILED, record.getWithdrawalAmount(), record.getChargeAmount(), record.getId());
			} else if(withdrawalStatusEnum.equals(WithdrawalStatusEnum.BACKOUT)){//撤销申请
				comResult = redMoneyRecordService.redMoneyUpdateRecord(AuthUtil.getUserId(), RedMoneySourceEnum.WITHDRAWAL_REVOKE, record.getWithdrawalAmount(), record.getChargeAmount(), record.getId());
			}
			if(ObjectUtil.isNotNull(comResult) && !comResult.getFlag()){
				throw new RuntimeException(comResult.getMsg());
			}
			//更新提现记录
			baseMapper.updateById(record);
			redisTool.unRedisLock("mallWithdrawalRecord:"+ id);
		} catch (Exception e){
			e.printStackTrace();
			throw e;
		} finally {
			redisTool.unRedisLock("mallWithdrawalRecord:"+ id);
		}


	}
}
