/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.modules.system.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springblade.common.cache.ParamCache;
import org.springblade.common.constant.CommonConstant;
import org.springblade.common.enums.MoneySourceEnum;
import org.springblade.common.enums.WithdrawalStatusEnum;
import org.springblade.common.utils.RedisTool;
import org.springblade.core.log.logger.BladeLogger;
import org.springblade.core.secure.BladeUser;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.modules.api.service.IAliService;
import org.springblade.modules.platform.service.UserAppService;
import org.springblade.modules.system.entity.*;
import org.springblade.modules.system.mapper.WithdrawalAccelerateRecordMapper;
import org.springblade.modules.system.mapper.WithdrawalRecordMapper;
import org.springblade.modules.system.param.RecordParam;
import org.springblade.modules.system.service.INowMoneyRecordService;
import org.springblade.modules.system.service.IUserService;
import org.springblade.modules.system.service.IWithdrawalRecordService;
import org.springblade.modules.system.vo.WithdrawalRecordVO;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
 * 服务实现类
 *
 * @author BladeX
 * @since 2022-07-18
 */
@Service
public class WithdrawalRecordServiceImpl extends ServiceImpl<WithdrawalRecordMapper, WithdrawalRecord> implements IWithdrawalRecordService {

	private static final Logger log = LoggerFactory.getLogger(INowMoneyRecordService.class);

	@Resource
	private BladeLogger logger;

	@Resource
	private UserAppService userAppService;

	@Resource
	private INowMoneyRecordService nowMoneyRecordService;

	@Resource
	private IAliService aliService;

	@Resource
	private IUserService userService;
	@Resource
	private RedisTool redisTool;

	@Resource
	private WithdrawalAccelerateRecordMapper accMapper;

	@Override
	public IPage<WithdrawalRecordVO> selectWithdrawalRecordPage(IPage<WithdrawalRecordVO> page, WithdrawalRecordVO withdrawalRecord) {
		return page.setRecords(baseMapper.selectWithdrawalRecordPage(page, withdrawalRecord));
	}

	@Override
	public Map<String, Object> createWithdrawalRecord(BigDecimal withdrawal, Integer isAccelerate) {
		BladeUser bladeUser = AuthUtil.getUser();
		User user = userService.getById(bladeUser.getUserId());
		LambdaQueryWrapper<UserApp> query = new LambdaQueryWrapper<>();
		query.eq(UserApp::getUserId, user.getId());
		query.eq(UserApp::getState, 1);
		UserApp userApp = userAppService.getOne(query);
		Map<String, Object> map = new HashMap<>();
		if (Objects.isNull(userApp)) {
			map.put("flag", false);
			map.put("msg", "钱包不存在或被冻结!");
			log.info(MoneySourceEnum.WITHDRAW.getName() + "失败", "用户钱包不存在或被冻结！");
			logger.info(MoneySourceEnum.WITHDRAW.getName() + "失败", "用户钱包不存在或被冻结！");
			return map;
		}

		BigDecimal chargeAmount = null;
		if (withdrawal.compareTo(new BigDecimal(20000)) <= 0) {
			chargeAmount = new BigDecimal(30);
		}
		if (withdrawal.compareTo(new BigDecimal(20000)) > 0) {
			chargeAmount = new BigDecimal(50);
		}

		if ((userApp.getNowMoney()
			.subtract(withdrawal)
			.subtract(chargeAmount))
			.compareTo(new BigDecimal(0)) == -1) {
			map.put("flag", false);
			map.put("msg", "钱包余额不足!");
			log.info(MoneySourceEnum.WITHDRAW.getName() + "失败", "用户钱包余额不足！");
			logger.info(MoneySourceEnum.WITHDRAW.getName() + "失败", "用户钱包余额不足！");
			return map;
		}

		LambdaQueryWrapper<WithdrawalRecord> queryWrapper = new LambdaQueryWrapper<>();
		queryWrapper.eq(WithdrawalRecord::getUserAppId, userApp.getId());
		queryWrapper.eq(WithdrawalRecord::getStatus, WithdrawalStatusEnum.TO_AUDIT.getValue());
		WithdrawalRecord record = this.getOne(queryWrapper);

		if (Objects.nonNull(record)) {
			map.put("flag", false);
			map.put("msg", "有提现申请待审核!");
			log.info(MoneySourceEnum.WITHDRAW.getName() + "失败", "该用户有提现申请待审核！");
			logger.info(MoneySourceEnum.WITHDRAW.getName() + "失败", "该用户有提现申请待审核！");
			return map;
		}

		WithdrawalRecord withdrawalRecord = new WithdrawalRecord();
		withdrawalRecord.setUserAppId(userApp.getId());
		withdrawalRecord.setWithdrawalAmount(withdrawal);
		withdrawalRecord.setChargeAmount(chargeAmount);
		withdrawalRecord.setStatus(WithdrawalStatusEnum.TO_AUDIT.getValue());
		withdrawalRecord.setCreateUserId(user.getId());
		withdrawalRecord.setCreateUserName(user.getRealName());
		withdrawalRecord.setUpdateUserId(user.getId());
		withdrawalRecord.setIsAccelerate(isAccelerate);
		Date date = new Date();
		withdrawalRecord.setCreateTime(date);
		withdrawalRecord.setUpdateTime(date);
		withdrawalRecord.setIsDelete(0);
		this.save(withdrawalRecord);

		RecordParam param = new RecordParam();
		param.setMoneySourceEnum(MoneySourceEnum.WITHDRAW);
		param.setFluctuantAmount(withdrawal);
		param.setChargeAmount(chargeAmount);
		param.setUserId(user.getId());
		param.setDataId(withdrawalRecord.getId());
		Map<String, Object> map1 = nowMoneyRecordService.insertRecord(param);

		if (!(Boolean) map1.get("flag")) {
			this.removeById(withdrawalRecord.getId());
		}

		return map1;
	}

	@Override
	@Transactional
	public Map<String, Object> updataWithdrawalRecord(Long withdrawalRecordId, WithdrawalStatusEnum statusEnum, String auditMsg) {

		try {

			LambdaQueryWrapper<WithdrawalRecord> query = new LambdaQueryWrapper<>();
			query.eq(WithdrawalRecord::getId, withdrawalRecordId);
			query.eq(WithdrawalRecord::getIsDelete, 0);
			WithdrawalRecord withdrawalRecord = this.getOne(query);
			Map<String, Object> map = new HashMap<>();
			if (Objects.isNull(withdrawalRecord)) {
				map.put("flag", false);
				map.put("msg", "提现申请不存在!");
				return map;
			}
			if (!redisTool.getRedisLock(withdrawalRecordId.toString(), withdrawalRecord.getCreateUserId().toString())) {
				throw new RuntimeException("该记录处理中，请稍后再试！");
			}

			if (withdrawalRecord.getStatus() != WithdrawalStatusEnum.TO_AUDIT.getValue()) {
				map.put("flag", false);
				map.put("msg", "提现申请不是待审核状态，禁止操作!");
				return map;
			}
			Date date = new Date();
			Map<String, Object> map1 = new HashMap<>();
			BladeUser bladeUser = AuthUtil.getUser();
			User user = userService.getById(bladeUser.getUserId());
			if (statusEnum == WithdrawalStatusEnum.REJECT) {
				RecordParam param = new RecordParam();
				param.setMoneySourceEnum(MoneySourceEnum.WITHDRAW_REJECT);
				param.setFluctuantAmount(withdrawalRecord.getWithdrawalAmount().add(withdrawalRecord.getChargeAmount()));
				param.setChargeAmount(new BigDecimal(0));
				param.setTaxAmount(new BigDecimal(0));
				param.setUserId(withdrawalRecord.getCreateUserId());
				param.setDataId(withdrawalRecord.getId());
				map1 = nowMoneyRecordService.insertRecord(param);

				if ((Boolean) map1.get("flag")) {
					withdrawalRecord.setStatus(statusEnum.getValue());
					withdrawalRecord.setAuditUserId(user.getId());
					withdrawalRecord.setAuditUserName(user.getRealName());
					withdrawalRecord.setAuditMsg(auditMsg);

					withdrawalRecord.setUpdateTime(date);
					withdrawalRecord.setAuditTime(date);
					this.saveOrUpdate(withdrawalRecord);
				}

			}

			if (statusEnum == WithdrawalStatusEnum.PASS) {
				UserApp userApp = userAppService.getById(withdrawalRecord.getUserAppId());
				NowMoneyRecord nowMoneyRecord = new NowMoneyRecord();
				nowMoneyRecord.setBeforeMoney(userApp.getNowMoney());
				nowMoneyRecord.setAfterMoney(userApp.getNowMoney());
				nowMoneyRecord.setType(MoneySourceEnum.WITHDRAW_PASS.getType());
				nowMoneyRecord.setSource(MoneySourceEnum.WITHDRAW_PASS.getName());
				nowMoneyRecord.setSourceCode(MoneySourceEnum.WITHDRAW_PASS.getValue());
				nowMoneyRecord.setFluctuantAmount(withdrawalRecord.getWithdrawalAmount());
				nowMoneyRecord.setChargeAmount(withdrawalRecord.getChargeAmount());
				nowMoneyRecord.setTaxAmount(new BigDecimal(0));
				nowMoneyRecord.setUserId(withdrawalRecord.getCreateUserId());
				nowMoneyRecord.setUserAppId(withdrawalRecord.getUserAppId());
				nowMoneyRecord.setParamStr(withdrawalRecord.toString());
				nowMoneyRecord.setDataId(withdrawalRecord.getId());
				nowMoneyRecord.setCreateUserId(user.getId());
				nowMoneyRecord.setUpdateUserId(user.getId());
				nowMoneyRecord.setCreateTime(date);
				nowMoneyRecord.setUpdateTime(date);
				nowMoneyRecord.setIsDelete(0);
				nowMoneyRecordService.save(nowMoneyRecord);
				withdrawalRecord.setStatus(statusEnum.getValue());
				withdrawalRecord.setAuditUserId(user.getId());
				withdrawalRecord.setAuditUserName(user.getRealName());
				withdrawalRecord.setAuditMsg(auditMsg);
				withdrawalRecord.setUpdateTime(date);
				withdrawalRecord.setAuditTime(date);
				this.saveOrUpdate(withdrawalRecord);

				if(withdrawalRecord.getIsAccelerate() == 1){
					WithdrawalAccelerateRecord record = new WithdrawalAccelerateRecord();
					BigDecimal amount = nowMoneyRecord.getFluctuantAmount();
					record.setRatio(new BigDecimal(ParamCache.getValue(CommonConstant.ACCELERATE_WITHDRAWAL_RATIO)));
					record.setDataId(withdrawalRecord.getId());
					record.setAmount(amount);
					record.setBeforeAmout(userApp.getAccBalance());
					BigDecimal sub = userApp.getAccBalance().subtract(amount);
					record.setAfterAmout(sub);
					record.setBenefitUserId(nowMoneyRecord.getUserId());
					record.setType(1);
					record.setSrc(2);
					record.setRemark("加速消耗");
					userApp.setAccBalance(sub);
					userAppService.updateById(userApp);
					accMapper.insert(record);
				}


				/**
				 * 向指定阿里用户付款
				 * @param type 类型默认0 余额提现 1 特殊账号打款 2 红包提现
				 * @param userId 系统用户Id
				 * @param aliUserId 阿里用户Id
				 * @param name 用户真实名字
				 * @param transAmount 金额
				 * @param orderTitle 标题
				 * @param remarks 备注
				 * @return PayLog
				 *
				 */
				//调支付
				User payUser = userService.getById(withdrawalRecord.getCreateUserId());
				PayLog payLog = aliService.pay(1,
					payUser.getId(),
					userApp.getAlipayUserId(),
					payUser.getRealName(),
					withdrawalRecord.getWithdrawalAmount(),
					"余额提现",
					"余额提现"
				);
				if (payLog.getIsSuccess() == 0) {
					throw new RuntimeException(payLog.getFailureReason());
				}

				map1.put("flag", true);
				map1.put("msg", payLog.getFailureReason());
			}

			return map1;
		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException("该记录处理中，请稍后再试！");
		} finally {//最后进行解锁
			redisTool.unRedisLock(withdrawalRecordId.toString());
		}
	}

	@Override
	public WithdrawalRecord getNewestByUserId(Long userId) {
		LambdaQueryWrapper<WithdrawalRecord> query = new LambdaQueryWrapper<>();
		query.eq(WithdrawalRecord::getIsDelete, 0);
		query.ne(WithdrawalRecord::getIsAccelerate, 1);
		query.eq(WithdrawalRecord::getStatus, WithdrawalStatusEnum.PASS.getValue());
		query.eq(WithdrawalRecord::getCreateUserId, userId);
		query.orderByDesc(WithdrawalRecord::getCreateTime);
		query.last("limit 1");
		return this.getOne(query);
	}

	@Override
	public Map<String, Object> backoutAudit(Long id) {

		try {

			Map<String, Object> map = new HashMap<>();
			BladeUser bladeUser = AuthUtil.getUser();
			WithdrawalRecord withdrawalRecord = this.getById(id);

			if (Objects.isNull(withdrawalRecord)) {
				map.put("flag", false);
				map.put("msg", "提现申请不存在！");
				return map;
			}
			if (!redisTool.getRedisLock(id.toString(), withdrawalRecord.getCreateUserId().toString())) {
				throw new RuntimeException("该记录处理中，请稍后再试！");
			}
			if (!bladeUser.getUserId().equals(withdrawalRecord.getCreateUserId())) {
				map.put("flag", false);
				map.put("msg", "禁止操作！");
				return map;
			}

			if (!withdrawalRecord.getStatus().equals(WithdrawalStatusEnum.TO_AUDIT.getValue())) {
				map.put("flag", false);
				map.put("msg", "禁止操作！");
				return map;
			}

			Date date = new Date();

			User user = userService.getById(bladeUser.getUserId());
			RecordParam param = new RecordParam();
			param.setMoneySourceEnum(MoneySourceEnum.WITHDRAW_BACKOUT);
			param.setFluctuantAmount(withdrawalRecord.getWithdrawalAmount().add(withdrawalRecord.getChargeAmount()));
			param.setChargeAmount(new BigDecimal(0));
			param.setTaxAmount(new BigDecimal(0));
			param.setUserId(withdrawalRecord.getCreateUserId());
			param.setDataId(withdrawalRecord.getId());
			map = nowMoneyRecordService.insertRecord(param);

			if ((Boolean) map.get("flag")) {
				withdrawalRecord.setStatus(WithdrawalStatusEnum.BACKOUT.getValue());
				withdrawalRecord.setAuditUserId(user.getId());
				withdrawalRecord.setAuditUserName(user.getRealName());
				withdrawalRecord.setAuditMsg("撤销提现申请");

				withdrawalRecord.setUpdateTime(date);
				withdrawalRecord.setAuditTime(date);
				this.saveOrUpdate(withdrawalRecord);

				map.put("flag", true);
				map.put("msg", "撤销成功！");
				return map;
			}
			return map;

		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException("该记录处理中，请稍后再试！");
		} finally {
			redisTool.unRedisLock(id.toString());
		}
	}

}
